/* REXX */
/*-------------------------------------------------------------------*/
/*
        NAME: CBROAMVT Version 1.0
 Description: Validates a CBROAMxx member and indicates any errors
              with syntax or values given for parameters. Mimics
              behavior of OAM initialization process, parsing the
              CBROAMxx member and providing the same error messages.

       Input: - Dataset name of CBROAMxx member
                Dataset name for output dataset
                YES/NO response for issuing of warning messages

      Output: - If any errors are detected, an output dataset will be
                allocated and written to. The dataset will contain
                error messages and wraning messages when applicable.
                Messages will be grouped by OAM set statements.

 Return Codes:
       0 - Successful completion, no errors found with CBROAMxx member
       2 - Successful completion, warnings found with CBROAMxx member
       4 - Successful completion, errors found with CBROAMxx member
       8 - Error with datasets provided
      12 - Program not running in foreground

  Written by: Veronica Reeves 05/03/2021

  CHANGE ACTIVITY:
  $01 II14776 121021 TUCVRV: CBROAMVT initial release             @01A

*/
/*-------------------------------------------------------------------*/
/* trace ?all */

arg parmin
call Initialize
if retcode = 0 then
 do
  "ALLOC F(DSIN) DS('"cbroam"') SHR"

  "EXECIO * DISKR DSIN (FINIS STEM LINE."
  if RC <> 0 then
   do
    say "Error reading CBROAMxx member "cbroam". Program ended."
    exit RC
  end

  call Quick_Pass

  /* if no explicit onlyif, continue on, otherwise ask for cross checking */
  if ex_onlyif <> 0 then
   do
    if words(parmin) <> 0 then
     do
      parse var parmin crosscheck parmin

      if crosscheck <> 'Y' & crosscheck <> 'N' then
       do
        retcode = 8
        say "Error: Value "crosscheck" is invalid for cross-checking."
        say "Program returned."
        return
      end

      if crosscheck = 'N' then
       do
        say "Only syntax checking will be done."
        ex_onlyif = 0
        syntax_only = 1
        first_onlyif = 1
      end
      else
       do
        parse var parmin mcbsysnm db2ssid oamvar parmin
        say 'Only statements for SYSNAME 'mcbsysnm', DB2ID ' db2ssid ||,
            ', and OAMVAR 'oamvar' will be checked.'
        mcbppign = 1
      end
    end
    else
     do
      response = ''
      do while response = ''
       say 'Explicit ONLYIF Statements found. Would you like to do cross-' ||,
           'checking? (Y/N)'
       PARSE UPPER PULL response
       if response <> 'Y' & response <> 'N' then
        do
         say "Please enter 'Y' or 'N'"
         response = ''
       end
      end

      if response = 'N' then
       do
        say "Only syntax checking will be done."
        ex_onlyif = 0
        syntax_only = 1
        first_onlyif = 1
        set_global_buf = 1
      end
      else
       do
        if set_sysname = 1 then
         do
          set_sysname = 0
          DO WHILE mcbsysnm = ''
           say "Enter SYSNAME: "
           parse pull mcbsysnm

           if mcbsysnm = '' then
            say "Field cannot be blank."
          END
          say "User input sysname: "mcbsysnm
        end

        if set_db2id = 1 then
         do
          set_db2id = 0
          DO WHILE db2ssid = ''
           say "Enter DB2ID: "
           parse pull db2ssid

           if db2ssid = '' then
            say "Field cannot be blank."
          END
          say "User input db2id: "db2ssid
        end

        if mcbppivr = 1 then
         do
          mcbppivr = 0
          DO WHILE oamvar = ''
           say "Enter OAMVAR: "
           parse pull oamvar

           if oamvar = '' then
            say "Field cannot be blank."
          END
          say "User input oamvar: "oamvar
        end

        mcbppign = 1 /* Set read stmt on */
      end
    end
  end

  aline = SUBSTR(LINE.linenum,1,72)

  /* Check for line continuation characters */
  end_chars = STRIP(SUBSTR(aline,71))
  if LENGTH(end_chars) = 1 then
   aline = STRIP(SUBSTR(aline,1,71))

  say "CBROAMVT tool running..."

  DO WHILE linenum <= LINE.0
   /* read in stmt */
   call Read_Stmt

   /* validate stmt */
   if current_stmt == statements.8 then
    do
     /* Check if any keywords present for the statement */
     if buf = '' then
      do
       QUEUE " "
       QUEUE COPIES("=",80)
       QUEUE CENTER(" "current_stmt" ",80,"-")
       QUEUE " "

       msgid = "0319I"
       issue_msg.0 = 2
       issue_msg.1 = current_stmt
       issue_msg.2 = cbroam

       call Write_Str
       QUEUE ' '

       warn_found = 1
       ITERATE
     end

     /* If doing only syntax checking, and have processed the first */
     /* ONLYIF statement, reset all the set statement parms. */
     if syntax_only = 1 & first_onlyif = 0 then
       call Set_Globals

     /* This is the first onlyif statement, set flag off */
     if first_onlyif = 1 then
      do
       set_global_buf = 0
       first_onlyif = 0
     end

     QUEUE " "
     QUEUE COPIES("=",80)
     QUEUE CENTER(" "current_stmt" "buf" ",80,"-")
     QUEUE " "

     call Validate_ONLYIF

     if err_found = 0 then
      do
       QUEUE "No errors found"
     end
     else
      do
       retcode = 4
     end
     QUEUE " "
   end
   else
    do
     if ex_onlyif = 0 | mcbppign = 1 then
      do
       QUEUE CENTER(" "current_stmt" ",80,"-")
       QUEUE " "

       /* Check if any keywords present for the statement */
       if buf = '' then
        do
         msgid = "0319I"
         issue_msg.0 = 2
         issue_msg.1 = current_stmt
         issue_msg.2 = cbroam

         call Write_Str
         QUEUE ' '

         warn_found = 1
         ITERATE
       end

       if set_global_buf = 1 then
        do
         stmt_num = global_buf.0 + 1
         global_buf.stmt_num.stmt = current_stmt
         global_buf.stmt_num.stmt_buf = buf
         global_buf.0 = stmt_num
       end

       select
        when current_stmt == statements.1 then
         call Validate_SETOAM
        when current_stmt == statements.2 then
         call Validate_SETOPT
        when current_stmt == statements.3 then
         call Validate_OAMXCF
        when current_stmt == statements.4 then
         call Validate_SETOSMC
        when current_stmt == statements.5 then
         call Validate_SETDISK
        when current_stmt == statements.6 then
         call Validate_SETTLIB
        when current_stmt == statements.7 then
         call Validate_SETCLOUD
        otherwise
         say "***UNKNOWN STATEMENT ERROR***"
       end

       if err_found = 0 then
        do
         QUEUE "No errors found"
       end
       else
        do
         retcode = 4
       end
       QUEUE " "
     end
   end
  END

  call End_Processing

  if warn_found <> 0 then
   do
    retcode = 2
  end

  if err_found <> 0 then
   do
    retcode = 4
  end

  "FREE F(DSIN)"
  say "CBROAMVT tool has ended"
  QUEUE ''


  if retcode = 0 then
   do
    say "No errors found with CBROAM member. Output dataset not allocated."
    DO QUEUED()
     PULL temp
    END
  end
  else
   do
    "ALLOC F(DSOUT) DS('"outds"') NEW"
    "EXECIO * DISKW DSOUT (FINIS"
    "FREE F(DSOUT)"
    if retcode = 4 then
     do
      say "Errors found with CBROAM member. Information written to "outds"."
    end

    if retcode = 2 then
     do
      say "Warning for with CBROAMxx member. No errors found. Information " ||,
          "written to "outds"."
    end
  end
end
exit

/**************************** Subroutines ****************************/
/*-------------------------------------------------------------------*/
/* Initialize the variables used by this exec                        */
/*-------------------------------------------------------------------*/
Initialize:

 /* control variables */
 debug = 0
 retcode = 0
 cbroam = ''
 outds = ''
 crosscheck = ''
 mcbsysnm = ''
 db2ssid = ''
 oamvar = ''

 /* ONLYIF control variables */
 ex_onlyif = 0
 syntax_only = 0
 first_onlyif = 0
 set_global_buf = 0
 global_buf.0 = 0 /* number of set stmts in the global space */
 mcbsysnm = ''
 db2ssid = ''
 oamvar = ''
 mcbppign = 1
 set_sysname = 0
 set_db2id = 0
 mcbppivr = 0

 /* CBROAM reading vars */
 linenum = 1
 read = 1
 buf = ''
 current_stmt = ''

 /* Types of set statements */
 statements.0 = 8
 statements.1 = "SETOAM"
 statements.2 = "SETOPT"
 statements.3 = "OAMXCF"
 statements.4 = "SETOSMC"
 statements.5 = "SETDISK"
 statements.6 = "SETTLIB"
 statements.7 = "SETCLOUD"
 statements.8 = "ONLYIF"

 /* CBROAM statement parsing vars */
 err_found = 0
 warn_found = 0
 ltparens = 0
 rtparens = 0

 /* Storage Group and XCF vars */
 current_sg = ''
 sg_ltparens = 0
 xcftimeout = 0
 vscb.0 = 0
 vscb_ptr = 0

 /* Message vars */
 msgid = 0
 issue_msg.0 = 0
 supress_msg = 0

 /* Type Check vars */
 A = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz"
 N = "0123456789"
 AN = A||N||"@$#"
 l2dir_validchars = A||N||"/_-."
 cont_validchars = A||N||"-."
 onlyif_validchars = AN||"*"

 call Message_List

 /* Initialize the global parameters */
 call Init_Globals

 say 'CBROAMVT v1.0: CBROAMxx verification tool'
 say '---------------------------------------------------------------'
 say 'If the given CBROAMxx dataset is found to have errors'
 say 'then an output data set will be created that contains'
 say 'the error messages associated with the parsing errors.'
 say 'This is a simulation of CBROAMxx processing. Initialization'
 say 'will not occur.'
 say '---------------------------------------------------------------'

 if words(parmin) > 0 then
  do
   if words(parmin) < 2 then
    do
     say 'Error: Missing some input parameters.'
     say 'Program returned.'
     retcode = 8
     return
    end

   parse var parmin cbroam outds parmin

   cbroam = STRIP(STRIP(cbroam,,"'"),,'"')
   outds = STRIP(STRIP(outds,,"'"),,'"')

   call Validate_Cbroam
   if retcode <> 0 then
    do
     retcode = 8
     say "Program returned."
     return
   end

   call Validate_OutDS
   if retcode <> 0 then
    do
     retcode = 8
     say "Program returned."
     return
   end

   if parmin = '' then
    parmin = 'N'
  end
 else
  do
   if debug = 0 then
    do
     if SYSVAR(SYSENV) = 'FORE' then
      do
       cbroam = ''
       do while cbroam = ''
        say 'Enter dataset name of CBROAMxx member to be validated:'
        pull cbroam
        if cbroam = '' then
         do
          say 'Field cannot be empty.'
        end
        else
         do
          call Validate_Cbroam
          if retcode = 4 then
           do
            say "Please enter a different dataset."
            cbroam = ''
            retcode = 0
          end
          if retcode = 8 then
           do
            say 'Program returned.'
            return
          end
        end
       end

       outds = ''
       do while outds = ''
        say 'Enter output data set name for validation result:'
        pull outds
        if outds = '' then
         do
          say 'Field cannot be empty.'
        end
        else
         do
          call Validate_OutDS
          if retcode = 4 then
           do
            say "Please enter a different dataset."
            outds = ''
            retcode = 0
          end
          if retcode = 8 then
           do
            say 'Program returned.'
            return
          end
        end
       end
     end
     else
      do
       say 'Error: Program is running in invalid enviornment.'
       retcode = 12
       return
     end
   end
 end

 return

/*-------------------------------------------------------------------*/
/* Routine: Validate_Cbroam                                          */
/* Purpose: Validate cbroamxx input dataset from user                */
/*-------------------------------------------------------------------*/
Validate_Cbroam:
 retcode = 0
 cbroam = STRIP(STRIP(cbroam,,"'"),,'"')

 say 'User input CBROAMxx dataset:' cbroam

 if SYSDSN("'"cbroam"'") <> 'OK' then
  do
   say "Input data set does not exist."
   retcode = 4
   return
 end

 response = LISTDSI("'"cbroam"'")
 if response <> 0 then
  do
   say 'Unexpected error from LISTDSI. Reason Code: 'SYSREASON
   retcode = 8
   return
 end
 if SYSDSSMS <> 'SEQ' then
  do
   if POS('(',cbroam) = 0 then
    do
     say 'Input data set is not a sequential data set.'
     retcode = 4
     return
   end
 end
 return

/*-------------------------------------------------------------------*/
/* Routine: Validate_OutDS                                           */
/* Purpose: Validate output dataset from user                        */
/*-------------------------------------------------------------------*/
Validate_OutDS:
 retcode = 0
 outds = STRIP(STRIP(outds,,"'"),,'"')

 say 'User output dataset name:' outds

 response = SYSDSN("'"outds"'")
 PARSE var response response extra

 if response = 'OK' then
  do
   say "Output data set '"outds"' already exists."
   retcode = 4
   return
 end

 if response = 'INVALID' then
  do
   say "Output data set '"outds"' is invalid."
   retcode = 4
   return
 end
 return

/*-------------------------------------------------------------------*/
/* Routine: Init_Globals                                             */
/* Purpose: Initialize the global parameters.                        */
/*-------------------------------------------------------------------*/
Init_Globals:
 /* SETOAM Global Parameters */
 setoam_stmt = 0 /* SETOAM Flag */
 mcbmtrt = 1     /* Global MAXTAPERETRIEVETASKS */
 mcbmtst = 1     /* Global MAXTAPESTORETASKS */
 mcbrcymt = 1    /* Global MAXRECYCLETASKS */

 /* OAMXCF Global Parameters */
 oamxcf_stmt = 0 /* OAMXCF Flag */
 mcbxcfgn = '' /* GROUPNAME */
 mcbxcfmn = '' /* MEMBERNAME */

 /* SETOSMC Global Parameters */
 mcbvscbb = 0 /* First Storage Backup Group */
 mcbvscb2 = 0 /* Second Storage Backup Group */
 mcbbk2on = 0 /* Indicates second backup group enabled */

 /* SETTLIB Global Parameters */
 ovtlarem = 0 /* LACSRETRYMINUTES */
 ovtlarts = 0 /* LACSRETRYTIMES */

 /* SETCLOUD Global Parameters */
 mcbcprov = '' /* Global PROVIDER Keyword */
 mcbccont = '' /* Global CONTAINER Keyword */

 return

/*-------------------------------------------------------------------*/
/* Routine: Add_SG                                                   */
/* Purpose: Initialize a new storage group and add it to the table.  */
/* Parms:                                                            */
/*   pass_sg - the name of the storage group to be added             */
/*-------------------------------------------------------------------*/
Add_SG:
 vscb.0 = vscb.0 + 1
 sg_ptr = vscb.0
 vscb.sg_ptr = pass_sg
 vscb.sg_ptr.type = ''

 call Init_SG

 return

/*-------------------------------------------------------------------*/
/* Routine: Init_SG                                                  */
/* Purpose: Reset a storage group to its initial values. Do not      */
/*          rest the storage group type.                             */
/* Parms:                                                            */
/*   sg_ptr - the index of the storage group in the vscb table       */
/*-------------------------------------------------------------------*/
Init_SG:
 vscb.sg_ptr.vsctstnt = 0  /* Modified by SETOAM */
 vscb.sg_ptr.vsctunit = '' /* TAPEUNITNAME */
 vscb.sg_ptr.vsc2tusp = 0  /* L2TAPEUNITNAME flag */
 vscb.sg_ptr.vsc2unit = '' /* L2TAPEUNITNAME */
 vscb.sg_ptr.vsctmtrt = 1  /* SG MAXTAPERETRIEVETACKS */
 vscb.sg_ptr.vscsgmtr = 0  /* SGMAXTAPERETRIEVETASKS flag */
 vscb.sg_ptr.vsctmtst = 1  /* SG MAXTAPESTORETASKES */
 vscb.sg_ptr.vscsgmts = 0  /* SGMAXTAPESTORETASKS flag */
 vscb.sg_ptr.vscrcymt = 1  /* SG MAXRECYCLETASKS */

 vscb.sg_ptr.setdisk = 0          /* Modified by SETDISK */
 vscb.sg_ptr.vscdl2dir = ''       /* L2DIRECTORY */
 vscb.sg_ptr.vscdstat_l2t_zfs = 0 /* L2TYPE ZFS flag */
 vscb.sg_ptr.vscdstat_l2t_nfs = 0 /* L2TYPE NFS flag */

 vscb.sg_ptr.1stbackup = 0 /* 1st backup storage group */
 vscb.sg_ptr.2ndbackup = 0 /* 2nd backup storage group */
 vscb.sg_ptr.bktype = 0    /* Backup group type */

 vscb.sg_ptr.vsccprov = '' /* Cloud provider */
 vscb.sg_ptr.vscccont = '' /* Cloud container */
 return

/*-------------------------------------------------------------------*/
/* Routine: Quick_Pass                                               */
/* Purpose: Process through the entire CBROAMxx member and determine */
/*          if there are any explicit ONLYIF statements.             */
/*          If there are ex_onlyif is set to 1.                      */
/*-------------------------------------------------------------------*/
Quick_Pass:
 aline = SUBSTR(LINE.linenum,1,72)

 /* Check for line continuation characters */
 end_chars = STRIP(SUBSTR(aline,71))
 if LENGTH(end_chars) = 1 then
  aline = STRIP(SUBSTR(aline,1,71))

 DO WHILE linenum <= LINE.0
  call Read_Stmt
  if current_stmt = statements.8 then
   do
  /*if POS('OAMVAR',buf) <> 0 | POS('*ALL*',buf) = 0 then*/
     if POS('SYSNAME',buf) <> 0 then
      set_sysname = 1
     if POS('DB2ID',buf) <> 0 then
      set_db2id = 1
     if POS('OAMVAR',buf) <> 0 then
      mcbppivr = 1
     ex_onlyif = 1
  end
 END

 linenum = 1
 read = 1
 keyword = 0

 /* Clear Queue */
 DO QUEUED()
  PULL temp
 END

 return

/*-------------------------------------------------------------------*/
/* Routine: Set_Globals                                              */
/* Purpose: To be used with syntax-only checking. If we have         */
/*          processed at least one ONLYIF statement, reinitialize    */
/*          all parameters and set the values given before any       */
/*          ONLYIF statement.                                        */
/*-------------------------------------------------------------------*/
Set_Globals:
 call End_Processing
 DO sg_ptr = 1 to vscb.0
  call Init_SG
 end
 vscb.0 = 0

 call Init_Globals

 /* Set the parameters given before the onlyif statements */
 supress_msg = 1
 onlyif_buf = buf
 i = 1

 DO WHILE i <= global_buf.0
  current_stmt = global_buf.i.stmt
  buf = global_buf.i.stmt_buf

  select
   when current_stmt == statements.1 then
    call Validate_SETOAM
   when current_stmt == statements.2 then
    call Validate_SETOPT
   when current_stmt == statements.3 then
    call Validate_OAMXCF
   when current_stmt == statements.4 then
    call Validate_SETOSMC
   when current_stmt == statements.5 then
    call Validate_SETDISK
   when current_stmt == statements.6 then
    call Validate_SETTLIB
   when current_stmt == statements.7 then
    call Validate_SETCLOUD
   otherwise
    say "***UNKNOWN STATEMENT ERROR***"
  end

  i = i + 1
 END

 current_stmt = "ONLYIF"
 buf = onlyif_buf
 supress_msg = 0
 return

/*-------------------------------------------------------------------*/
/* Routine: Read_Stmt                                                */
/* Purpose: Read in a full statement from CBROAMxx member into       */
/*          a buffer.                                                */
/*          Finds first instance of a set statement defined in       */
/*          'statements' list and reads all following data until the */
/*          next set statement is found. Comments and extra          */
/*          whitespace are ignored.                                  */
/*-------------------------------------------------------------------*/
Read_Stmt:
 current_stmt = ''
 buf = ''
 start = linenum

 DO WHILE linenum <= LINE.0
  DO WHILE aline <> ''
   PARSE VAR aline temp aline

   if read then
    do
     /* determine if there are any comments */
     cmt_open = POS('/*', temp)
     if cmt_open <> 0 then
      do
       cmt_close = POS('*/',temp)
       if cmt_close <> 0 then
        temp = SUBSTR(temp,1,cmt_open - 1) || SUBSTR(temp,cmt_close + 2)
       else
        do
        /* stop reading until end of comment found */
        temp = SUBSTR(temp,1,cmt_open - 1)
        read = 0
       end
     end

     /* Determine if there are any system symbols to replace */
     if POS('&',temp) <> 0 then
      call Replace_SysSymbol

     /* determine if current word is a set stmt */
     if temp == statements.1 | temp == statements.2,
      | temp == statements.3 | temp == statements.4,
      | temp == statements.5 | temp == statements.6,
      | temp == statements.7 | temp == statements.8 then
      do
       if current_stmt == '' then
        do
         current_stmt = temp
       end
       else
        do
        /* new set stmt found, end read if not still on starting line */
        if linenum <> start then
         do
          aline = temp aline
          buf = STRIP(buf)
          return
        end
        else
         buf = buf temp
       end
     end
     else
      do
       if current_stmt == '' & temp <> '' then
        do
         /* Missing first set stmt */
         QUEUE "ERROR unknown set stmt: '"temp"'"
       end
       else
        /* add to buffer */
        buf = buf temp
     end
   end
   else
    do
     /* determine if end of comment found */
     cmt_close = POS('*/', temp)
     if cmt_close <> 0 then
      do
       aline = SUBSTR(temp,cmt_close + 2) || aline
       read = 1
     end
   end
  END /* WHILE aline <> '' */

  linenum = linenum + 1
  aline = SUBSTR(LINE.linenum,1,72)

  /* Check for line continuation characters */
  end_chars = STRIP(SUBSTR(aline,71))
  if LENGTH(end_chars) = 1 then
   aline = STRIP(SUBSTR(aline,1,71))

 END /* WHILE linenum <= LINE.0 */

 buf = STRIP(buf)
 return

/*-------------------------------------------------------------------*/
/* Routine: Parse_Buf                                                */
/* Purpose: Parse a keyword/value pair from the buffer.              */
/*          Loops through the control structure for the current set  */
/*          statement and use the length of the parameters and the   */
/*          delimitter in the structure to determine which keyword   */
/*          was found.                                               */
/*          Once a keyword is identified, the value is parsed from   */
/*          the buffer and the syntax is checked against the values  */
/*          in the structure.                                        */
/*          Errors such as missing end parentheses, extra data for   */
/*          keywords, and invalid keywords are also dealt with here  */
/*-------------------------------------------------------------------*/
Parse_Buf:
 /* Determine if last keyword had ending parentheses */
 if ltparens <> 0 then
  do
   sclparm = "INV_END"
   return
 end

 if sg_ltparens <> 0 | xcftimeout = 1 then
  do
   if SUBSTR(buf,1,1) = ')' then
    do
     sclparm = "SG_RTPARENS"
     return
   end
 end

 if current_sg <> '' | xcftimeout = 1 then
  struct = "SG_CTRL_STRUCT"
 else
  struct = "CTRL_STRUCT"

 /* Find next keyword */
 DO i = 1 to structs.struct.0
  brk = LENGTH(structs.struct.i) + 1

  /* Go through keyword delims and determine which occurs first */
  DO j = 1 to structs.struct.i.2.0
   delim = POS(structs.struct.i.2.j, buf)

   if delim <> 0 & delim <= brk then
    do
     brk = delim

     if structs.struct.i.2.j = '(' then
      ltparens = 1
     else
      ltparens = 0
   end
  END

  /* Take substring up to delimitter for comparison */
  comp = SUBSTR(buf,1,brk - 1)
  if comp = structs.struct.i,
   | comp = SUBSTR(structs.struct.i,1,structs.struct.i.1) then
   /*process keyword */
   do
    keyword = 1
    sclparm = structs.struct.i
    scluindx = i

    buf = STRIP(SUBSTR(buf,brk + 1))
    if ltparens = 0 & structs.struct.scluindx.3 <> "NONE" then
     do
      if sclparm <> "STORAGEGROUP" then
       do
        msgid = "0302I"
        issue_msg.0 = 2
        issue_msg.1 = sclparm
        issue_msg.2 = current_stmt

        call Write_Str
        QUEUE ' '

        sclparm = "SYNTAX_ERR"
        LEAVE
      end
    end

    if ltparens <> 0  & structs.struct.scluindx.3 <> "NONE" then
     do
      sp = POS(' ',buf)
      cm = POS(',',buf)
      rp = POS(')',buf)
      brk = LENGTH(buf) + 1

      if sp <> 0 then
       brk = sp
      if cm <> 0 & cm < brk then
       brk = cm
      if rp <> 0 & rp <= brk then
       do
        brk = rp
        rtparens = 1
      end

      val = SUBSTR(buf,1,brk - 1)
      buf = STRIP(SUBSTR(buf,brk + 1))

      /* Value is too long */
      if LENGTH(val) < structs.struct.scluindx.5,
       | (structs.struct.scluindx.6 > 0,
       & LENGTH(val) > structs.struct.scluindx.6) then
       do
        if LENGTH(val) = 0 then
         do
          msgid = "0306I"
          issue_msg.0 = 2
          issue_msg.1 = sclparm
          issue_msg.2 = current_stmt

          call Write_Str
          QUEUE ' '

          sclparm = "SYNTAX_ERR"
        end
        else
         do

          msgid = "0303I"
          issue_msg.0 = 3
          issue_msg.1 = sclparm
          issue_msg.2 = current_stmt
          issue_msg.3 = val

          call Write_Str

          msgid = "ADD"
          issue_msg.0 = 1
          issue_msg.1 = "Data is invalid length."

          call Write_Str
          QUEUE ' '

          sclparm = "SYNTAX_ERR"
        end
        if rtparens = 1 then
         do
          rtparens = 0
          ltparens = 0
        end
        LEAVE
      end
      else
       do
        /* Value is wrong type */
        if structs.struct.scluindx.4 <> '' & sclparm <> "SYNTAX_ERR" then
         do
          if VERIFY(SUBSTR(val,1,1),VALUE(structs.struct.scluindx.4)) <> 0
           then
           do
            msgid = "0303I"
            issue_msg.0 = 3
            issue_msg.1 = sclparm
            issue_msg.2 = current_stmt
            issue_msg.3 = val

            call Write_Str

            msgid = "ADD"
            issue_msg.0 = 1
            issue_msg.1 = "First character of data is invalid type."

            call Write_Str
            QUEUE ' '

            sclparm = "SYNTAX_ERR"
          end
        end
        if structs.struct.scluindx.3 <> '' & sclparm <> "SYNTAX_ERR" then
         do
          if VERIFY(val,VALUE(structs.struct.scluindx.3)) <> 0 then
           do
            msgid = "0303I"
            issue_msg.0 = 3
            issue_msg.1 = sclparm
            issue_msg.2 = current_stmt
            issue_msg.3 = val

            call Write_Str

            msgid = "ADD"
            issue_msg.0 = 1
            issue_msg.1 = "Data is invalid type."

            call Write_Str
            QUEUE ' '

            sclparm = "SYNTAX_ERR"
          end
        end

        if rtparens = 1 then
         do
          rtparens = 0
          ltparens = 0
        end
        LEAVE
      end
    end
   LEAVE
  end
  else
   do
    ltparens = 0
  end
 END /* DO i = 1 to structs.struct */
 if keyword = 0 then
  do
   sclparm = "INV_KEYWD"
 end
 return

/*-------------------------------------------------------------------*/
/* Routine: Write_Str                                                */
/* Purpose: Parses a string into 80 character (max) substrings to    */
/*          queue on output buffer to be written to the output data  */
/*          set later.                                               */
/*-------------------------------------------------------------------*/
Write_Str:
 if supress_msg = 1 then
  return

 i = 1
 astr = CBR.msgid
 inpos = POS("*insert*",astr)

 DO WHILE inpos <> 0
  astr = INSERT(issue_msg.i,DELSTR(astr,inpos,8),inpos - 1)
  i = i + 1
  inpos = POS("*insert*",astr)
 end

 DO WHILE astr <> ''
  PARSE VALUE astr with L 81 R
  if POS(' ',R) <> 1 & R <> '' then
   do
    sp = LASTPOS(' ',L)
    if sp <> 0 then
     do
      R = SUBSTR(L,sp+1)R
      L = SUBSTR(L,1,sp-1)
    end
  end
  astr = STRIP(R)
  QUEUE L
 END
 return

/*-------------------------------------------------------------------*/
/* Routine: Replace_SysSymbol                                        */
/* Purpose: Identify and replace the system symbol in the varibale   */
/*          temp with the correct value, if applicable.              */
/*-------------------------------------------------------------------*/
Replace_SysSymbol:
 amp = POS('&',temp)
 amp_i = 8

 /* Identify the name of the system symbol */
 DO UNTIL amp_i = 0
  amp_temp = STRIP(SUBSTR(temp,amp+1,amp_i))
  if VERIFY(amp_temp,AN) = 0 then
   do
    /* Replace the system symbol if it exists */
    syssym = MVSVAR('SYMDEF',SUBSTR(temp,amp+1,amp_i))
    if syssym <> '' then
     temp = SUBSTR(temp,1,amp-1)||syssym||SUBSTR(temp,amp+amp_i+1)
    LEAVE
   end
  amp_i = amp_i - 1
 end

 return

/*-------------------------------------------------------------------*/
/* Routine: End_Processing                                           */
/* Purpose: Check variables after all parsing has finished.          */
/*          Determine which set statements were specified and        */
/*          check any of the dependent variables for them.           */
/*          This allows the tool to mimic the double pass that the   */
/*          parser does during OAM initialization.                   */
/*-------------------------------------------------------------------*/
End_Processing:
 err_found = 0

 QUEUE CENTER("FINAL PARAMETER CHECK",80,"-")
 QUEUE " "

 if oamxcf_stmt = 1 then
  do
   if mcbxcfgn = '' then
    do
     msgid = "0329I"
     issue_msg.0 = 1
     issue_msg.1 = cbroam

     call Write_Str
     QUEUE ' '

     err_found = 1
   end

   if mcbxcfmn = '' then
    do
     msgid = "0330I"
     issue_msg.0 = 1
     issue_msg.1 = cbroam

     call Write_Str
     QUEUE ' '

     err_found = 1
   end
 end

 if (mcbcprov <> '' & mcbccont = '') |,
    (mcbcprov = '' & mcbccont <> '') then
  do
   msgid = "0363I"
   issue_msg.0 = 3
   issue_msg.1 = statements.7
   issue_msg.2 = "PROVIDER"
   issue_msg.2 = "CONTAINER"

   call Write_Str
   QUEUE ' '


   warn_found = 1
   mcbcprov = ''
   mcbccont = ''
 end

 /* Check Storage Groups */

 v = 1
 b_count = 0
 b2_count = 0

 DO WHILE v <= vscb.0
  if vscb.v.vsctstnt = 1 then
   do
    if vscb.v.vsctunit = '' then
     do
      msgid = "0311I"
      issue_msg.0 = 2
      issue_msg.1 = "TAPEUNITNAME"
      issue_msg.2 = vscb.v

      call Write_Str

      msgid = "ADD"
      issue_msg.0 = 1
      issue_msg.1 = "Message CBR0311I applies to a SETOAM statement."

      call Write_Str
      QUEUE ' '

      err_found = 1
    end
    if vscb.v.vsc2tusp = 1 & vscb.v.vsc2unit = '' then
     do
      msgid = "0311I"
      issue_msg.0 = 2
      issue_msg.1 = "L2TAPEUNITNAME"
      issue_msg.2 = vscb.v

      call Write_Str

      msgid = "ADD"
      issue_msg.0 = 1
      issue_msg.1 = "Message CBR0311I applies to a SETOAM statement."

      call Write_Str
      QUEUE ' '

      err_found = 1
    end
    if vscb.v.vsctmtrt > mcbmtrt then
     do
      msgid = "0313I"
      issue_msg.0 = 4
      issue_msg.1 = vscb.v

      if vscb.v.vscsgmtr = 0 then
       issue_msg.2 = "MAXTAPERETRIEVETASKS"
      else
       issue_msg.2 = "SGMAXTAPERETRIEVETASKS"

      issue_msg.3 = vscb.v.vsctmtrt
      issue_msg.4 = mcbmtrt

      call Write_Str
      QUEUE ' '

      err_found = 1
    end
    if vscb.v.vsctmtst > mcbmtst then
     do
      msgid = "0314I"
      issue_msg.0 = 4
      issue_msg.1 = vscb.v

      if vscb.v.vscsgmts = 0 then
       issue_msg.2 = "MAXTAPESTORETASKS"
      else
       issue_msg.2 = "SGMAXTAPESTORETASKS"

      issue_msg.3 = vscb.v.vsctmtst
      issue_msg.4 = mcbmtst

      call Write_Str
      QUEUE ' '

      err_found = 1
    end
    if vscb.v.vscrcymt > mcbrcymt then
     do
      msgid = "0342I"
      issue_msg.0 = 3
      issue_msg.1 = vscb.v
      issue_msg.2 = vscb.v.vscrcymt
      issue_msg.3 = mcbrcymt

      call Write_Str
      QUEUE ' '

      err_found = 1
    end
  end

  if vscb.v.type = 'B' then
   do
    b_count = b_count + 1
    if vscb.v.bktype = 2 then
     b2_count = b2_count + 1
  end

  if vscb.v.type = '' | vscb.v.type = 'G' | vscb.v.type ='B' then
   do
    if vscb.v.vsccprov = '' & mcbcprov <> '' then
     vscb.v.vsccprov = mcbcprov
    if vscb.v.vscccont = '' & mcbccont <> '' then
     vscb.v.vscccont = mcbccont
    if vscb.v.vsccprov <> '' && vscb.v.vscccont <> '' then
     do
      if vscb.v.vsccprov <> '' | vscb.v.vscccont <> '' |,
         vscb.v.vscccrto <> -1 | vscb.v.vscccwto <> -1 then
       do
        msgid = "0362I"
        issue_msg.0 = 4
        issue_msg.1 = statements.7
        issue_msg.2 = vscb.v
        issue_msg.3 = "PROVIDER"
        issue_msg.4 = "CONTAINER"

        call Write_Str
        QUEUE ' '


        warn_found = 1
        vscb.v.vsccprov = ''
        vscb.v.vscccont = ''
        vscb.v.vscccrto = mcbccrto
        vscb.v.vscccwto = mcbccwto
      end
    end
  end

  if (vscb.sg_ptr.vscdstat_l2t_zfs = 1 | vscb.sg_ptr.vscdstat_l2t_nfs = 1) &&,
     vscb.sg_ptr.vscdl2dir <> '' then
   do
    msgid = "0349I"
    issue_msg.0 = 1
    issue_msg.1 = vscb.v

    call Write_Str
    QUEUE ' '

    err_found = 1
  end
  v = v + 1
 END

 if b2_count > 0 & b2_count = b_count then
  do
   msgid = "0337I"
   issue_msg.0 = 1
   issue_msg.1 = cbroam

   call Write_Str
   QUEUE ' '

   err_found = 1
 end
 if err_found = 0 then
  do
   QUEUE "No errors found"
 end
 else
  do
   retcode = 4
 end
 QUEUE " "
 return

/*-------------------------------------------------------------------*/
/* Routine: Validate_SETOAM                                          */
/* Purpose: Validate the SETOAM statement.                           */
/*          Loops throught the buffer of keywords and values, using  */
/*          Parse_Buf to extract keyword/value pairs. Check the      */
/*          values and issue error messages for invalid values given.*/
/*-------------------------------------------------------------------*/
Validate_SETOAM:

 call SETOAM_STRUCT
 err_found = 0

 setoam_stmt = 1 /* Indicate SETOAM statement present */

 DO WHILE buf <> ''
  call Parse_Buf

  SELECT
   /* NO-Value Keyword Processing */
   /* Global DSNWITHSGNAME and Storage Group TAPECOMPACTION & NOTAPECOMPATION */
   WHEN (current_sg = '' & sclparm = structs.struct.11) |,
        (current_sg <> '' & (sclparm = structs.struct.2 |,
         sclparm = structs.struct.3)) then
    do
     keyword = 0
   end
   /* Value Keyword Processing */
   /* SYSZTIOTUSEXTIOT*/
   WHEN current_sg = '' & sclparm = structs.struct.1 then
    do
     keyword = 0
     if val <> "ENABLED" & val <> "DISABLED" then
      do
       msgid = "0303I"
       issue_msg.0 = 3
       issue_msg.1 = sclparm
       issue_msg.2 = current_stmt
       issue_msg.3 = val

       call Write_Str

       msgid = "ADD"
       issue_msg.0 = 1
       issue_msg.1 = "Value should be 'ENABLED' or 'DISABLED'."

       call Write_Str
       QUEUE ' '

       err_found = 1
     end
   end

   /* Global DISABLEOPENTYPEJ */
   WHEN current_sg = '' & sclparm = structs.struct.2 then
    do
     keyword = 0
     if val <> "INOUT" & val <> "NO" then
      do
       msgid = "0303I"
       issue_msg.0 = 3
       issue_msg.1 = sclparm
       issue_msg.2 = current_stmt
       issue_msg.3 = val

       call Write_Str

       msgid = "ADD"
       issue_msg.0 = 1
       issue_msg.1 = "Value should be 'INOUT' or 'NO'."

       call Write_Str
       QUEUE ' '

       err_found = 1
     end
   end

   /* Global MAXTAPERETRIEVETASKS and MAXTAPESTORETASKS */
   WHEN current_sg = '' & (sclparm = structs.struct.3 |,
        sclparm = structs.struct.4) then
    do
     keyword = 0
     if val <= 0 | val > 100 then
      do
       msgid = "0303I"
       issue_msg.0 = 3
       issue_msg.1 = sclparm
       issue_msg.2 = current_stmt
       issue_msg.3 = val

       call Write_Str

       msgid = "ADD"
       issue_msg.0 = 1
       issue_msg.1 = "Value should not be less than 1 or greater than 100."

       call Write_Str
       QUEUE ' '

       err_found = 1
     end
     else
      do
       if sclparm = structs.struct.3 then
        mcbmtrt = val
       else
        mcbmtst = val
     end
   end

   /* Global MOUNTWAITTIME */
   WHEN current_sg = '' & sclparm = structs.struct.5 then
    do
     keyword = 0
     if val = 0 | val > 120 then
      do
       msgid = "0303I"
       issue_msg.0 = 3
       issue_msg.1 = sclparm
       issue_msg.2 = current_stmt
       issue_msg.3 = val

       call Write_Str

       msgid = "ADD"
       issue_msg.0 = 1
       issue_msg.1 = "Value should not be less than 1 or greater than 120."

       call Write_Str
       QUEUE ' '

       err_found = 1
     end
   end

   /* Global DEMOUNTWAITTIME */
   WHEN current_sg = '' & sclparm = structs.struct.6 then
    do
     keyword = 0
     if val <= 0 | val > 9999 then
      do
       msgid = "0303I"
       issue_msg.0 = 3
       issue_msg.1 = sclparm
       issue_msg.2 = current_stmt
       issue_msg.3 = val

       call Write_Str

       msgid = "ADD"
       issue_msg.0 = 1
       issue_msg.1 = "Value should not be less than 1 or greater than 9999."

       call Write_Str
       QUEUE ' '

       err_found = 1
     end
   end

   /* Global TAPESDB */
   WHEN current_sg = '' & sclparm = structs.struct.7 then
    do
     keyword = 0
     if val <> 'LARGE' & val <> 'SMALL' then
      do
       msgid = "0303I"
       issue_msg.0 = 3
       issue_msg.1 = sclparm
       issue_msg.2 = current_stmt
       issue_msg.3 = val

       call Write_Str

       msgid = "ADD"
       issue_msg.0 = 1
       issue_msg.1 = "Value should be 'LARGE' or 'SMALL'."

       call Write_Str
       QUEUE ' '

       err_found = 1
     end
   end

   /* Global ALLOCRETRYMINUTES */
   WHEN current_sg = '' & sclparm = structs.struct.8 then
    do
     keyword = 0
     if val < 0 | val > 5 then
      do
       msgid = "0303I"
       issue_msg.0 = 3
       issue_msg.1 = sclparm
       issue_msg.2 = current_stmt
       issue_msg.3 = val

       call Write_Str

       msgid = "ADD"
       issue_msg.0 = 1
       issue_msg.1 = "Value should not be less than 0 or greater than 5."

       call Write_Str
       QUEUE ' '

       err_found = 1
     end
   end

   /* Global TAPEFULLTHRESHOLD */
   WHEN current_sg = '' & sclparm = structs.struct.9 then
    do
     keyword = 0
     if val < 0 then
      do
       msgid = "0303I"
       issue_msg.0 = 3
       issue_msg.1 = sclparm
       issue_msg.2 = current_stmt
       issue_msg.3 = val

       call Write_Str

       msgid = "ADD"
       issue_msg.0 = 1
       issue_msg.1 = "Value should not be less than 0."

       call Write_Str
       QUEUE ' '

       err_found = 1
     end
   end

   /* Global TAPEEXPIRATION */
   WHEN current_sg = '' & sclparm = structs.struct.10 then
    do
     keyword = 0
     year = SUBSTR(val,1,4)
     slash = SUBSTR(val,5,1)
     day = SUBSTR(val,6,3)

     if DATATYPE(year,'N') & DATATYPE(day,'N') & slash = '/' then
      do
       if day = 0 | day > 366 then
        do
         msgid = "0303I"
         issue_msg.0 = 3
         issue_msg.1 = sclparm
         issue_msg.2 = current_stmt
         issue_msg.3 = val

         call Write_Str

         msgid = "ADD"
         issue_msg.0 = 1
         issue_msg.1 = "Value for day should not be less than 1 or greater " ||,
                       "than 366."

         call Write_Str
         QUEUE ' '

         err_found = 1
       end
       else
        do
         if year < 1900 | year > 2155 then
          do
           msgid = "0303I"
           issue_msg.0 = 3
           issue_msg.1 = sclparm
           issue_msg.2 = current_stmt
           issue_msg.3 = val

           call Write_Str

           msgid = "ADD"
           issue_msg.0 = 1
           issue_msg.1 = "Value for year should not be less than 1900 or " ||,
                         "greater than 2155."

           call Write_Str
           QUEUE ' '

           err_found = 1
         end
         else
          do
           if day = 366 then
            do
             if year % 4 <> 0 | (year % 100 = 0 & year % 400 <> 0) then
              do
               msgid = "0303I"
               issue_msg.0 = 3
               issue_msg.1 = sclparm
               issue_msg.2 = current_stmt
               issue_msg.3 = val

               call Write_Str

               msgid = "ADD"
               issue_msg.0 = 1
               issue_msg.1 = "Value for day is 366 but the year is not a " ||,
                             "leap year."

               call Write_Str
               QUEUE ' '

               err_found = 1
             end
           end
         end
       end
     end
     else
      do
       msgid = "0303I"
       issue_msg.0 = 3
       issue_msg.1 = sclparm
       issue_msg.2 = current_stmt
       issue_msg.3 = val

       call Write_Str

       msgid = "ADD"
       issue_msg.0 = 1
       issue_msg.1 = "Value should be in the format 'yyyy/ddd'."

       call Write_Str
       QUEUE ' '

       err_found = 1
     end
   end

   /* Global DATACLASS and L2DATACLASS */
   WHEN current_sg = '' & (sclparm = structs.struct.12 |,
        sclparm = structs.struct.13) then
    do
     keyword = 0
   end

   /* STORAGEGROUP name */
   WHEN current_sg = '' & sclparm = structs.struct.14 then
    do
     keyword = 0
     current_sg = val
     sg_ltparens = 1
     ltparens = 0
     vscb_ptr = 0

     DO i = 1 to vscb.0
      if vscb.i = current_sg then
       do
        vscb_ptr = i
        LEAVE
      end
     END
     if vscb_ptr = 0 then
      do
       pass_sg = current_sg
       call Add_SG
       vscb_ptr = vscb.0
     end

     vscb.vscb_ptr.vsctstnt = 1
   end

   /* Global TAPECAPACITY */
   WHEN current_sg = '' & sclparm = structs.struct.15 then
    do
     keyword = 0

     if ltparens <> 0 then
      do
       brk = LENGTH(buf)
       sp = POS(' ',buf)
       cm = POS(',',buf)
       rp = POS(')',buf)
       if sp <= brk & sp > 0 then
        brk = sp
       if cm <= brk & cm > 0 then
        brk = cm
       if rp <= brk & rp > 0 then
        do
         brk = rp
         ltparens = 0
       end

       hold_capacity = SUBSTR(buf,1,brk-1)
       buf = STRIP(SUBSTR(buf,brk+1))

       if LENGTH(hold_capacity) < 1 | LENGTH(hold_capacity) > 10 then
        do
         if LENGTH(hold_capacity) = 0 then
          do
           msgid = "0306I"
           issue_msg.0 = 2
           issue_msg.1 = sclparm
           issue_msg.2 = current_stmt

           call Write_Str
           QUEUE ' '

           err_found = 1
         end
         else
          do
           msgid = "0303I"
           issue_msg.0 = 3
           issue_msg.1 = sclparm
           issue_msg.2 = current_stmt
           issue_msg.3 = hold_capacity

           call Write_Str

           msgid = "ADD"
           issue_msg.0 = 1
           issue_msg.1 = "Data is invalid length."

           call Write_Str
           QUEUE ' '

           err_found = 1
         end
       end
       else
        do
         /* Value is wrong type */
         if \DATATYPE(hold_capacity, 'N') then
          do
           msgid = "0303I"
           issue_msg.0 = 3
           issue_msg.1 = sclparm
           issue_msg.2 = current_stmt
           issue_msg.3 = hold_capacity

           call Write_Str

           msgid = "ADD"
           issue_msg.0 = 1
           issue_msg.1 = "Data is invalid type."

           call Write_Str
           QUEUE ' '

           err_found = 1
         end
       end

       if hold_capacity <= 0 | hold_capacity > 2147483647 then
        do
         msgid = "0303I"
         issue_msg.0 = 3
         issue_msg.1 = sclparm
         issue_msg.2 = current_stmt
         issue_msg.3 = hold_capacity

         call Write_Str

         msgid = "ADD"
         issue_msg.0 = 1
         issue_msg.1 = "Value for tape capacity should be greater than 0 " ||,
                       "and less than 21474483647."

         call Write_Str
         QUEUE ' '

         err_found = 1
       end
     end
     else
      do
       msgid = "0303I"
       issue_msg.0 = 3
       issue_msg.1 = sclparm
       issue_msg.2 = current_stmt
       issue_msg.3 = val

       call Write_Str

       msgid = "ADD"
       issue_msg.0 = 1
       issue_msg.1 = "Both a tape capacity name and a tape capacity must " ||,
                     "be specified."

       call Write_Str
       QUEUE ' '

       err_found = 1
     end
   end

   /* Global TAPERECYCLEMODE */
   WHEN current_sg = '' & sclparm = structs.struct.16 then
    do
     keyword = 0
     if val <> "GROUP" & val <> "MVSSCRATCH" &,
        val <> "OAMSCRATCH" then
      do
       msgid = "0303I"
       issue_msg.0 = 3
       issue_msg.1 = sclparm
       issue_msg.2 = current_stmt
       issue_msg.3 = val

       call Write_Str

       msgid = "ADD"
       issue_msg.0 = 1
       issue_msg.1 = "Value should be 'GROUP' 'MVSSCRATCH' or 'OAMSCRATCH'."

       call Write_Str
       QUEUE ' '

       err_found = 1
     end
   end

   /* Global OAMSCRATCHSYNCH */
   WHEN current_sg = '' & sclparm = structs.struct.17 then
    do
     keyword = 0
     if val <> "ENABLED" & val <> "DISABLED" then
      do
       msgid = "0303I"
       issue_msg.0 = 3
       issue_msg.1 = sclparm
       issue_msg.2 = current_stmt
       issue_msg.3 = val

       call Write_Str

       msgid = "ADD"
       issue_msg.0 = 1
       issue_msg.1 = "Value should be 'ENABLED' or 'DISABLED'."

       call Write_Str
       QUEUE ' '

       err_found = 1
     end
   end

   /* Global TAPEDISPATCHERDELAY */
   WHEN current_sg = '' & sclparm = structs.struct.18 then
    do
     keyword = 0
     if val < 1 | val > 60 then
      do
       msgid = "0303I"
       issue_msg.0 = 3
       issue_msg.1 = sclparm
       issue_msg.2 = current_stmt
       issue_msg.3 = val

       call Write_Str

       msgid = "ADD"
       issue_msg.0 = 1
       issue_msg.1 = "Value should not be less than 1 or greater than 60."

       call Write_Str
       QUEUE ' '

       err_found = 1
     end
   end

   /* Global MAXRECYCLETASKS */
   WHEN current_sg = '' & sclparm = structs.struct.19 then
    do
     keyword = 0
     if val > 15 then
      do
       msgid = "0303I"
       issue_msg.0 = 3
       issue_msg.1 = sclparm
       issue_msg.2 = current_stmt
       issue_msg.3 = val

       call Write_Str

       msgid = "ADD"
       issus_msg.0 = 1
       issue_msg.1 = "Value should not be greater than 15."

       call Write_Str
       QUEUE ' '

       err_found = 1
     end
     else
      do
       mcbrcymt = val
     end
   end

   /* Global PERCENTVALID */
   WHEN current_sg = '' & sclparm = structs.struct.20 then
    do
     keyword = 0
     if val > 100 then
      do
       msgid = "0303I"
       issue_msg.0 = 3
       issue_msg.1 = sclparm
       issue_msg.2 = current_stmt
       issue_msg.3 = val

       call Write_Str

       msgid = "ADD"
       issue_msg.0 = 1
       issue_msg.1 = "Value should not be greater than 100."

       call Write_Str
       QUEUE ' '

       err_found = 1
     end
   end

   /* Storage Group TAPEUNITNAME and L2TAPEUNITNAME */
   WHEN current_sg <> '' & (sclparm = structs.struct.1 |,
        sclparm = structs.struct.16) then
    do
     keyword = 0
     if vscb_ptr <> 0 then
      do
       if sclparm = structs.struct.1 then
        vscb.vscb_ptr.vsctunit = val
       else
        do
         vscb.vscb_ptr.vsc2tusp = 1
         vscb.vscb_ptr.vsc2unit = val
       end
     end
   end

   /* Storage Group MAXTAPERETRIEVETASKS, MAXTAPESTORETASKS,
      SGMAXTAPERETRIEVETASKS, and SGMAXTAPESTORETASKS */
   WHEN current_sg <> '' & (sclparm = structs.struct.4 |,
        sclparm = structs.struct.5 | sclparm = structs.struct.6 |,
        sclparm = structs.struct.7) then
    do
     keyword = 0
     if vscb_ptr <> 0 then
      do
       if val = 0 | val > 100 then
        do
         msgid = "0303I"
         issue_msg.0 = 3
         issue_msg.1 = sclparm
         issue_msg.2 = current_stmt
         issue_msg.3 = val

         call Write_Str

         msgid = "ADD"
         issue_msg.0 = 1
         issue_msg.1 = "Value should not be less than 1 or greater than 100."

         call Write_Str
         QUEUE ' '

         err_found = 1
       end
       else
        do
         if sclparm = structs.struct.4 | sclparm = structs.struct.6 then
          do
           vscb.vscb_ptr.vsctmtrt = val
           if sclparm = structs.struct.4 then
            vscb.vscb_ptr.vscsgmtr = 0
           else
            vscb.vscb_ptr.vscsgmtr = 1
         end
         else
          do
           vscb.vscb_ptr.vsctmtst = val
           if sclparm = structs.struct.5 then
            vscb.vscb_ptr.vscsgmts = 0
           else
            vscb.vscb_ptr.vscsgmts = 1
         end
       end
     end
   end

   /* Storage Group SGMAXRECYCLETASKS */
   WHEN current_sg <> '' & sclparm = structs.struct.8 then
    do
     keyword = 0
     if vscb_ptr <> 0 then
      do
       if val > 15 then
        do
         msgid = "0303I"
         issue_msg.0 = 3
         issue_msg.1 = sclparm
         issue_msg.2 = current_stmt
         issue_msg.3 = val

         call Write_Str

         msgid = "ADD"
         issue_msg.0 = 1
         issue_msg.1 = "Value should not be greater than 15."

         call Write_Str
         QUEUE ' '

         err_found = 1
       end
       else
        do
         vscb.vscb_ptr.vscrcymt = val
       end
     end
   end

   /* Storage Group TAPEDRIVESTARTUP */
   WHEN current_sg <> '' & sclparm = structs.struct.9 then
    do
     keyword = 0
     if vscb_ptr <> 0 then
      do
       if val = 0 | val > 9999 then
        do
         msgid = "0303I"
         issue_msg.0 = 3
         issue_msg.1 = sclparm
         issue_msg.2 = current_stmt
         issue_msg.3 = val

         call Write_Str

         msgid = "ADD"
         issue_msg.0 = 1
         issue_msg.1 = "Value should not be less than 1 or greater than 9999."

         call Write_Str
         QUEUE ' '

         err_found = 1
       end
     end
   end

   /* Storage Group TAPEPERCENTFULL */
   WHEN current_sg <> '' & sclparm = structs.struct.10 then
    do
     keyword = 0
     if vscb_ptr <> 0 then
      do
       if val = 0 | val > 100 then
        do
         msgid = "0303I"
         issue_msg.0 = 3
         issue_msg.1 = sclparm
         issue_msg.2 = current_stmt
         issue_msg.3 = val

         call Write_Str

         msgid = "ADD"
         issue_msg.0 = 1
         issue_msg.1 = "Value should not be less than 1 or greater than 100."

         call Write_Str
         QUEUE ' '

         err_found = 1
       end
     end
   end

   /* Storage Group DEMOUNTWAITTIME */
   WHEN current_sg <> '' & sclparm = structs.struct.11 then
    do
     keyword = 0
     if vscb_ptr <> 0 then
      do
       if val <= 0 | val > 9999 then
        do
         msgid = "0303I"
         issue_msg.0 = 3
         issue_msg.1 = sclparm
         issue_msg.2 = current_stmt
         issue_msg.3 = val

         call Write_Str

         msgid = "ADD"
         issue_msg.0 = 1
         issue_msg.1 = "Value should not be less than 1 or greater than 9999."

         call Write_Str
         QUEUE ' '

         err_found = 1
       end
     end
   end

   /* Storage Group TAPEFULLTHESHOLD */
   WHEN current_sg <> '' & sclparm = structs.struct.12 then
    do
     keyword = 0
     if vscb_ptr <> 0 then
      do
       if val < 0 then
        do
         msgid = "0303I"
         issue_msg.0 = 3
         issue_msg.1 = sclparm
         issue_msg.2 = current_stmt
         issue_msg.3 = val

         call Write_Str

         msgid = "ADD"
         issue_msg.0 = 1
         issue_msg.1 = "Value should not be less than 0."

         call Write_Str
         QUEUE ' '

         err_found = 1
       end
     end
   end

   /* Storage Group TAPEEXPIRATION */
   WHEN current_sg <> '' & sclparm = structs.struct.13 then
    do
     keyword = 0
     if vscb_ptr <> 0 then
      do
       year = SUBSTR(val,1,4)
       slash = SUBSTR(val,5,1)
       day = SUBSTR(val,6,3)

       if DATATYPE(year,'N') & DATATYPE(day,'N') & slash = '/' then
        do
         if day = 0 | day > 366 then
          do
           msgid = "0303I"
           issue_msg.0 = 3
           issue_msg.1 = sclparm
           issue_msg.2 = current_stmt
           issue_msg.3 = val

           call Write_Str

           msgid = "ADD"
           issue_msg.0 = 1
           issue_msg.1 = "Value for day should not be less than 1 or " ||,
                         "greater than 366."

           call Write_Str
           QUEUE ' '

           err_found = 1
         end
         else
          do
           if year < 1900 | year > 2155 then
            do
             msgid = "0303I"
             issue_msg.0 = 3
             issue_msg.1 = sclparm
             issue_msg.2 = current_stmt
             issue_msg.3 = val

             call Write_Str

             msgid = "ADD"
             issue_msg.0 = 1
             issue_msg.1 = "Value for year should not be less than 1900 or " ||,
                           "greater than 2155."

             call Write_Str
             QUEUE ' '

             err_found = 1
           end
           else
            do
             if day = 366 then
              do
               if year % 4 <> 0 | (year % 100 = 0 & year % 400 <> 0) then
                do
                 msgid = "0303I"
                 issue_msg.0 = 3
                 issue_msg.1 = sclparm
                 issue_msg.2 = current_stmt
                 issue_msg.3 = val

                 call Write_Str

                 msgid = "ADD"
                 issue_msg.0 = 1
                 issue_msg.1 = "Value for day is 366 but the year is not a " ||,
                               "leap year."

                 call Write_Str
                 QUEUE ' '

                 err_found = 1
               end
             end
           end
         end
       end
       else
        do
         msgid = "0303I"
         issue_msg.0 = 3
         issue_msg.1 = sclparm
         issue_msg.2 = current_stmt
         issue_msg.3 = val

         call Write_Str

         msgid = "ADD"
         issue_msg.0 = 1
         issue_msg.1 = "Value should be in the format 'yyyy/ddd'."

         call Write_Str
         QUEUE ' '

         err_found = 1
       end
     end
   end

   /* Storage Group DATACLASS L2DATACLASS */
   WHEN current_sg <> '' & (sclparm = structs.struct.14 |,
        sclparm = structs.struct.15) then
    do
     keyword = 0
   end

   /* Ending parenthesis for STORAGEGROUP parameter */
   WHEN sclparm = "SG_RTPARENS" then
    do
     sg_ltparens = 0
     current_sg = ''
     vscb_ptr = 0
     buf = STRIP(SUBSTR(buf,2))
   end

   /* ENDING CHARACTER NOT A ')' (I.E. - EMBEDDED BLANK OR COMMA) */
   WHEN sclparm = "INV_END" then
    do
     keyword = 0
     ltparens = 0
     brk = POS(")",buf)
     if brk = 0 then
      brk = LENGTH(buf)
     extra = STRIP(SUBSTR(buf,1,brk))
     buf = STRIP(SUBSTR(buf,brk + 1))

     if LENGTH(extra) > 1 then
      do
       msgid = "0304I"
       issue_msg.0 = 3
       issue_msg.1 = structs.struct.scluindx
       issue_msg.2 = current_stmt
       issue_msg.3 = extra

       call Write_Str
       QUEUE ' '

       err_found = 1
     end
   end

   /*  KEYWORD VALUE SYNTAX INCORRECT                                  */
   /*                                                                  */
   /*  BAD SYNTAX CAN BE ONE OF THE FOLLOWING:                         */
   /*                                                                  */
   /*  - THE LENGTH OF THE DATA IS WRONG.  FOR EXAMPLE, THE LENGTH     */
   /*    SHOULD BE BETWEEN 1 AND 3, AND THE CUSTOMER HAS ENTERED DATA  */
   /*    WITH A LENGTH OF 4.                                           */
   /*  - THE VALUE IS SUPPOSED TO BE NUMERIC, AND THERE IS AN          */
   /*    ALPHABETIC CHARACTER IN IT.                                   */
   /*  - THE DATA IS SUPPOSED TO START WITH A LETTER, AND IT STARTS    */
   /*    WITH A NUMBER                                                 */
   /*  - THE DATA IS SUPPOSED TO ONLY BE LETTERS AND NUMBERS, AND IT   */
   /*    HAS A SPECIAL CHARACTER IN IT                                 */
   WHEN sclparm = "SYNTAX_ERR" then
    do
     keyword = 0
     err_found = 1
   end

   /* Unknown keyword or keyword starting with left parens */
   WHEN sclparm = "INV_KEYWD" then
    do
     /* Unknown Keyword */
     if POS('(',buf) = 1 then
      do
       msgid = "0302I"
       issue_msg.0 = 2
       issue_msg.1 = '('
       issue_msg.2 = current_stmt

       call Write_Str
       QUEUE ' '

       buf = SUBSTR(buf,2)
     end
     else
      do
       brk = LENGTH(buf) + 1
       sp = POS(" ",buf)
       cm = POS(",",buf)
       rp = POS(")",buf)
       if sp <> 0 then
        brk = sp
       if cm <> 0 & cm < brk then
        brk = cm
       if rp <> 0 & rp < brk then
        do
         ltparens = 0
         brk = rp
       end
       brk = brk + 1

       extra = STRIP(SUBSTR(buf,1,brk - 1))
       buf = STRIP(SUBSTR(buf,brk))

       msgid = "0302I"
       issue_msg.0 = 2
       issue_msg.1 = extra
       issue_msg.2 = current_stmt

       call Write_Str
       QUEUE ' '

     end
     err_found = 1
   end
   OTHERWISE
    say "Unknown Error "sclparm" in SETOAM SG '"current_sg"'"
  END /* SELECT */
  keyword = 0
 END

 /* Last keyword was missing end parenthesis */
 if ltparens <> 0 then
  do
   msgid = "0303I"
   issue_msg.0 = 3
   issue_msg.1 = sclparm
   issue_msg.2 = current_stmt
   issue_msg.3 = val

   call Write_Str

   msgid = "ADD"
   issue_msg.0 = 1
   issue_msg.1 = "Missing end parenthesis."

   call Write_Str
   QUEUE ' '

   ltparens = 0
   err_found = 1
 end

 /* Last STORAGEGROUP missing end parenthesis */
 if sg_ltparens <> 0 then
  do
   msgid = "0305I"
   issue_msg.0 = 2
   issue_msg.1 = current_sg
   issue_msg.2 = current_stmt

   call Write_Str
   QUEUE ' '

   sg_ltparens = 0
   err_found = 1
   current_sg = ''
   vscb_ptr = 0
 end

 /* Last keyword missing data */
 if keyword <> 0 then
  do
   msgid = "0306I"
   issue_msg.0 = 2
   issue_msg.1 = sclparm
   issue_msg.2 = current_stmt

   call Write_Str
   QUEUE ' '

   err_found = 1
 end
 return

/*-------------------------------------------------------------------*/
/* Routine: Validate_SETOPT                                          */
/* Purpose: Validate the SETOPT statement.                           */
/*          Loops throught the buffer of keywords and values, using  */
/*          Parse_Buf to extract keyword/value pairs. Check the      */
/*          values and issue error messages for invalid values given.*/
/*-------------------------------------------------------------------*/
Validate_SETOPT:

 call SETOPT_STRUCT
 err_found = 0

 DO WHILE buf <> ''
  call Parse_Buf

  SELECT
   /* Global OPTICALREINITMODE */
   WHEN current_sg = '' & sclparm = structs.struct.1 then
    do
     keyword = 0
     if val <> "GROUP" & val <> "OAMSCRATCH" then
      do
       msgid = "0303I"
       issue_msg.0 = 3
       issue_msg.1 = sclparm
       issue_msg.2 = current_stmt
       issue_msg.3 = val

       call Write_Str

       msgid = "ADD"
       issue_msg.0 = 1
       issue_msg.1 = "Value should be 'GROUP' or 'OAMSCRATCH'."

       call Write_Str
       QUEUE ' '

       err_found = 1
     end
   end

   /* Global ABALL, ABDB2ERROR, ABLOST, ABNOTOPER, ABOFFLINE, ABUNREAD,
      ABFFSERROR, and ABCLDERROR */
   WHEN current_sg = '' & (sclparm = structs.struct.2 |,
        sclparm = structs.struct.3 | sclparm = structs.struct.4 |,
        sclparm = structs.struct.5 | sclparm = structs.struct.6 |,
        sclparm = structs.struct.7 | sclparm = structs.struct.8 |,
        sclparm = structs.struct.9) then
    do
     keyword = 0
     if val <> "BACKUP1" & val <> "BACKUP2" & val <> "INACTIVE" then
      do
       msgid = "0303I"
       issue_msg.0 = 3
       issue_msg.1 = sclparm
       issue_msg.2 = current_stmt
       issue_msg.3 = val

       call Write_Str

       msgid = "ADD"
       issue_msg.0 = 1
       issue_msg.1 = "Value should be 'BACKUP1' 'BACKUP2' or 'INACTIVE'."

       call Write_Str
       QUEUE ' '

       err_found = 1
     end
   end

   /* Global MULTISYSENABLE */
   WHEN current_sg = '' & sclparm = structs.struct.10 then
    do
     keyword = 0
     if val <> "YES" & val <> "NO" then
      do
       msgid = "0303I"
       issue_msg.0 = 3
       issue_msg.1 = sclparm
       issue_msg.2 = current_stmt
       issue_msg.3 = val

       call Write_Str

       msgid = "ADD"
       issue_msg.0 = 1
       issue_msg.1 = "Value should be 'YES' or 'NO'."

       call Write_Str
       QUEUE ' '

       err_found = 1
     end
   end

   /* Global OPTICALDISPATCHERDELAY */
   WHEN current_sg = '' & sclparm = structs.struct.11 then
    do
     keyword = 0
     if val < 1 | val > 60 then
      do
       msgid = "0303I"
       issue_msg.0 = 3
       issue_msg.1 = sclparm
       issue_msg.2 = current_stmt
       issue_msg.3 = val

       call Write_Str

       msgid = "ADD"
       issue_msg.0 = 1
       issue_msg.1 = "Value should not be less than 1 or greater than 60."

       call Write_Str
       QUEUE ' '

       err_found = 1
     end
   end

   /* Global SCRENTRYTHRESHOLD */
   WHEN current_sg = '' & sclparm = structs.struct.12 then
    do
     keyword = 0
     if val < 0 | val > 99999 then
      do
       msgid = "0303I"
       issue_msg.0 = 3
       issue_msg.1 = sclparm
       issue_msg.2 = current_stmt
       issue_msg.3 = val

       call Write_Str

       msgid = "ADD"
       issue_msg.0 = 1
       issue_msg.1 = "Value should not be less than 0 or greater than 9999."

       call Write_Str
       QUEUE ' '

       err_found = 1
     end
   end

   /* Global MOUNTWAITTIME */
   WHEN current_sg = '' & sclparm = structs.struct.13 then
    do
     keyword = 0
     if val = 0 | val > 120 then
      do
       msgid = "0303I"
       issue_msg.0 = 3
       issue_msg.1 = sclparm
       issue_msg.2 = current_stmt
       issue_msg.3 = val

       call Write_Str

       msgid = "ADD"
       issue_msg.0 = 1
       issue_msg.1 = "Value should not be less than 1 or greater than 120."

       call Write_Str
       QUEUE ' '

       err_found = 1
     end
   end

   /* Global UNLOADDRIVES */
   WHEN current_sg = '' & sclparm = structs.struct.14 then
    do
     keyword = 0
     if val = 0 | val > 6 then
      do
       msgid = "0303I"
       issue_msg.0 = 3
       issue_msg.1 = sclparm
       issue_msg.2 = current_stmt
       issue_msg.3 = val

       call Write_Str

       msgid = "ADD"
       issue_msg.0 = 1
       issue_msg.1 = "Value should not be less than 1 or greater than 6."

       call Write_Str
       QUEUE ' '

       err_found = 1
     end
   end

   /* Global UNLOADTIMER */
   WHEN current_sg = '' & sclparm = structs.struct.15 then
    do
     keyword = 0
     if val = 0 | val > 9999 then
      do
       msgid = "0303I"
       issue_msg.0 = 3
       issue_msg.1 = sclparm
       issue_msg.2 = current_stmt
       issue_msg.3 = val

       call Write_Str

       msgid = "ADD"
       issue_msg.0 = 1
       issue_msg.1 = "Value should not be less than 1 or greater than 9999."

       call Write_Str
       QUEUE ' '

       err_found = 1
     end
   end

   /* STORAGEGROUP Name */
   WHEN current_sg = '' & sclparm = structs.struct.16 then
    do
     keyword = 0
     current_sg = val
     sg_ltparens = 1
     ltparens = 0
     vscb_ptr = 0

     DO i = 1 to vscb.0
      if vscb.i = current_sg then
       do
        vscb_ptr = i
        LEAVE
      end
     END
     if vscb_ptr = 0 then
      do
       pass_sg = current_sg
       call Add_SG
       vscb_ptr = vscb.0
     end
   end

   /* Global DB2RECONNECTWAITTIME */
   WHEN current_sg = '' & sclparm = structs.struct.17 then
    do
     keyword = 0
     if val < 10 | val > 300 then
      do
       msgid = "0303I"
       issue_msg.0 = 3
       issue_msg.1 = sclparm
       issue_msg.2 = current_stmt
       issue_msg.3 = val

       call Write_Str

       msgid = "ADD"
       issue_msg.0 = 1
       issue_msg.1 = "Value should not be less than 10 or greater than 300."

       call Write_Str
       QUEUE ' '

       err_found = 1
     end
   end

   /* Storage Group OPTICALREINTMODE */
   WHEN current_sg <> '' & sclparm = structs.struct.1 then
    do
     keyword = 0
     if vscb_ptr <> 0 then
      do
       if val <> "GROUP" & val <> "OAMSCRATCH" then
        do
         msgid = "0303I"
         issue_msg.0 = 3
         issue_msg.1 = sclparm
         issue_msg.2 = current_stmt
         issue_msg.3 = val

         call Write_Str

         msgid = "ADD"
         issue_msg.0 = 1
         issue_msg.1 = "Value should be 'GROUP' or 'OAMSCRATCH'."

         call Write_Str
         QUEUE ' '

         err_found = 1
       end
     end
   end

   /* Ending parenthesis for STORAGEGROUP parameter */
   WHEN sclparm = "SG_RTPARENS" then
    do
     sg_ltparens = 0
     current_sg = ''
     vscb_ptr = 0
     buf = STRIP(SUBSTR(buf,2))
   end

   /* ENDING CHARACTER NOT A ')' (I.E. - EMBEDDED BLANK OR COMMA) */
   WHEN sclparm = "INV_END" then
    do
     keyword = 0
     ltparens = 0
     brk = POS(")",buf)
     if brk = 0 then
      brk = LENGTH(buf)
     extra = STRIP(SUBSTR(buf,1,brk))
     buf = STRIP(SUBSTR(buf,brk + 1))

     if LENGTH(extra) > 1 then
      do
       msgid = "0304I"
       issue_msg.0 = 3
       issue_msg.1 = structs.struct.scluindx
       issue_msg.2 = current_stmt
       issue_msg.3 = extra

       call Write_Str
       QUEUE ' '

       err_found = 1
     end
   end

   /*  KEYWORD VALUE SYNTAX INCORRECT                                  */
   /*                                                                  */
   /*  BAD SYNTAX CAN BE ONE OF THE FOLLOWING:                         */
   /*                                                                  */
   /*  - THE LENGTH OF THE DATA IS WRONG.  FOR EXAMPLE, THE LENGTH     */
   /*    SHOULD BE BETWEEN 1 AND 3, AND THE CUSTOMER HAS ENTERED DATA  */
   /*    WITH A LENGTH OF 4.                                           */
   /*  - THE VALUE IS SUPPOSED TO BE NUMERIC, AND THERE IS AN          */
   /*    ALPHABETIC CHARACTER IN IT.                                   */
   /*  - THE DATA IS SUPPOSED TO START WITH A LETTER, AND IT STARTS    */
   /*    WITH A NUMBER                                                 */
   /*  - THE DATA IS SUPPOSED TO ONLY BE LETTERS AND NUMBERS, AND IT   */
   /*    HAS A SPECIAL CHARACTER IN IT                                 */
   WHEN sclparm = "SYNTAX_ERR" then
    do
     keyword = 0
     err_found = 1
   end

   /* Unknown keyword or keyword starting with left parenthesis */
   WHEN sclparm = "INV_KEYWD" then
    do
     /* Unknown Keyword */
     if POS('(',buf) = 1 then
      do
       msgid = "0302I"
       issue_msg.0 = 2
       issue_msg.1 = "("
       issue_msg.2 = current_stmt

       call Write_Str
       QUEUE ' '

       buf = SUBSTR(buf,2)
     end
     else
      do
       brk = LENGTH(buf) + 1
       sp = POS(" ",buf)
       cm = POS(",",buf)
       rp = POS(")",buf)
       if sp <> 0 then
        brk = sp
       if cm <> 0 & cm < brk then
        brk = cm
       if rp <> 0 & rp < brk then
        do
         ltparens = 0
         brk = rp
       end
       brk = brk + 1

       extra = STRIP(SUBSTR(buf,1,brk - 1))
       buf = STRIP(SUBSTR(buf,brk))

       msgid = "0302I"
       issue_msg.0 = 2
       issue_msg.1 = extra
       issue_msg.2 = current_stmt

       call Write_Str
       QUEUE ' '
     end
     err_found = 1
   end
   OTHERWISE
     say "Unknown Error "sclparm" in SETOPT SG "current_sg
  END /* SELECT */
  keyword = 0
 END /* DO WHILE buf <> '' */

 /* Last keyword was missing end parenthesis */
 if ltparens <> 0 then
   do
    msgid = "0303I"
    issue_msg.0 = 3
    issue_msg.1 = sclparm
    issue_msg.2 = current_stmt
    issue_msg.3 = val

    call Write_Str

    msgid = "ADD"
    issue_msg.0 = 1
    issue_msg.1 = "Missing end parenthesis."

    call Write_Str
    QUEUE ' '

    ltparens = 0
    err_found = 1
 end

 /* Last STORAGEGROUP missing end parenthesis */
 if sg_ltparens <> 0 then
  do
   msgid = "0305I"
   issue_msg.0 = 2
   issue_msg.1 = current_sg
   issue_msg.2 = current_stmt

   call Write_Str
   QUEUE ' '

   sg_ltparens = 0
   err_found = 1
   current_sg = ''
   vscb_ptr = 0
 end

 /* Last keyword missing data */
 if keyword <> 0 then
  do
   msgid = "0306I"
   issue_msg.0 = 2
   issue_msg.1 = sclparm
   issue_msg.2 = current_stmt

   call Write_Str
   QUEUE ' '

   err_found = 1
 end
 return

/*-------------------------------------------------------------------*/
/* Routine: Validate_OAMXCF                                          */
/* Purpose: Validate the OAMXCF statement.                           */
/*          Loops throught the buffer of keywords and values, using  */
/*          Parse_Buf to extract keyword/value pairs. Check the      */
/*          values and issue error messages for invalid values given.*/
/*-------------------------------------------------------------------*/
Validate_OAMXCF:

 call OAMXCF_STRUCT
 err_found = 0

 oamxcf_stmt = 1 /* Indicate OAMXCF statement present */

 DO WHILE buf <> ''
  call Parse_Buf

  SELECT
   /* Global XCFTIMEOUT */
   WHEN xcftimeout = 0 & sclparm = structs.struct.1 then
    do
     keyword = 0
     xcftimeout = 1
     ltparens = 0
   end

   /* Global OAMGROUPNAME */
   WHEN xcftimeout = 0 & sclparm = structs.struct.2 then
    do
     keyword = 0
     mcbxcfgn = val
   end

   /* Global OAMMEMBERNAME */
   WHEN xcftimeout = 0 & sclparm = structs.struct.3 then
    do
     keyword = 0
     mcbxcfmn = val
   end

   /* XCFTIMEOUT parameters XCFOPTREADA, XCFOPTREADM, XCFOPTWRITEA,
      XCFOPTWRITEM, XCFTAPEREADA, and XCFTAPEREADM */
   WHEN xcftimeout = 1 & (sclparm = structs.struct.1 |,
        sclparm = structs.struct.2 | sclparm = structs.struct.3 |,
        sclparm = structs.struct.4 | sclparm = structs.struct.5 |,
        sclparm = structs.struct.6 ) then
    do
     keyword = 0

     if val < 1 | val > 999999 then
      do
       msgid = "0303I"
       issue_msg.0 = 3
       issue_msg.1 = sclparm
       issue_msg.2 = current_stmt
       issue_msg.3 = val

       call Write_Str

       msgid = "ADD"
       issue_msg.0 = 1
       issue_msg.1 = "Value should not be less than 1 or greater than 999999."

       call Write_Str
       QUEUE ' '

       err_found = 1
     end
   end

   /* Ending parenthesis for XCFTIMEOUT */
   WHEN sclparm = "SG_RTPARENS" then
    do
     xcftimeout = 0
     buf = STRIP(SUBSTR(buf,2))
   end

   /* ENDING CHARACTER NOT A ')' (I.E. - EMBEDDED BLANK OR COMMA) */
   WHEN sclparm = "INV_END" then
    do
     keyword = 0
     ltparens = 0
     brk = POS(")",buf)
     if brk = 0 then
      brk = LENGTH(buf)
     extra = STRIP(SUBSTR(buf,1,brk))
     buf = STRIP(SUBSTR(buf,brk + 1))

     if LENGTH(extra) > 1 then
      do
       msgid = "0304I"
       issue_msg.0 = 3
       issue_msg.1 = structs.struct.scluindx
       issue_msg.2 = current_stmt
       issue_msg.3 = extra

       call Write_Str
       QUEUE ' '

       err_found = 1
     end
   end

   /*  KEYWORD VALUE SYNTAX INCORRECT                                  */
   /*                                                                  */
   /*  BAD SYNTAX CAN BE ONE OF THE FOLLOWING:                         */
   /*                                                                  */
   /*  - THE LENGTH OF THE DATA IS WRONG.  FOR EXAMPLE, THE LENGTH     */
   /*    SHOULD BE BETWEEN 1 AND 3, AND THE CUSTOMER HAS ENTERED DATA  */
   /*    WITH A LENGTH OF 4.                                           */
   /*  - THE VALUE IS SUPPOSED TO BE NUMERIC, AND THERE IS AN          */
   /*    ALPHABETIC CHARACTER IN IT.                                   */
   /*  - THE DATA IS SUPPOSED TO START WITH A LETTER, AND IT STARTS    */
   /*    WITH A NUMBER                                                 */
   /*  - THE DATA IS SUPPOSED TO ONLY BE LETTERS AND NUMBERS, AND IT   */
   /*    HAS A SPECIAL CHARACTER IN IT                                 */
   WHEN sclparm = "SYNTAX_ERR" then
    do
     keyword = 0
     err_found = 1
   end

   /* Unknown keyword or keyword starting with left parenthesis */
   WHEN sclparm = "INV_KEYWD" then
    do
     /* Unknown Keyword */
     if POS('(',buf) = 1 then
      do
       msgid = "0302I"
       issue_msg.0 = 2
       issue_msg.1 = '('
       issue_msg.2 = current_stmt

       call Write_Str
       QUEUE ' '

       buf = SUBSTR(buf,2)
     end
     else
      do
       brk = LENGTH(buf) + 1
       sp = POS(" ",buf)
       cm = POS(",",buf)
       rp = POS(")",buf)
       if sp <> 0 then
        brk = sp
       if cm <> 0 & cm < brk then
        brk = cm
       if rp <> 0 & rp < brk then
        do
         ltparens = 0
         brk = rp
       end
       brk = brk + 1

       extra = STRIP(SUBSTR(buf,1,brk - 1))
       buf = STRIP(SUBSTR(buf,brk))

       msgid = "0302I"
       issue_msg.0 = 2
       issue_msg.1 = extra
       issue_msg.2 = current_stmt

       call Write_Str
       QUEUE ' '

     end
     err_found = 1
   end
   OTHERWISE
     say "Unknown Error "sclparm" in OAMXCF SG '"current_sg"'"
  end /* SELECT */
  keyword = 0
 END /* DO WHILE buf <> '' */

 /* Last keyword missing end parenthesis */
 if ltparens <> 0 then
  do
   msgid = "0303I"
   issue_msg.0 = 3
   issue_msg.1 = sclparm
   issue_msg.2 = current_stmt
   issue_msg.3 = val

   call Write_Str

   msgid = "ADD"
   issue_msg.0 = 1
   issue_msg.1 = "Missing end parenthesis."

   call Write_Str
   QUEUE ' '

   ltparens = 0
   err_found = 1
 end

 /* Last XCFTIMEOUT missing end parenthesis */
 if xcftimeout <> 0 then
  do
   msgid = "0326I"
   issue_msg.0 = 0

   call Write_Str
   QUEUE ' '

   xcftimeout = 0
   err_found = 1
 end

 /* Last keyword missing data */
 if keyword <> 0 then
  do
   msgid = "0306I"
   issue_msg.0 = 2
   issue_msg.1 = sclparm
   issue_msg.2 = current_stmt

   call Write_Str
   QUEUE ' '

   err_found = 1
 end

 return

/*-------------------------------------------------------------------*/
/* Routine: Validate_SETOSMC                                         */
/* Purpose: Validate the SETOSMC statement.                          */
/*          Loops throught the buffer of keywords and values, using  */
/*          Parse_Buf to extract keyword/value pairs. Check the      */
/*          values and issue error messages for invalid values given.*/
/*-------------------------------------------------------------------*/
Validate_SETOSMC:

 call SETOSMC_STRUCT
 err_found = 0

 DO WHILE buf <> ''
  call Parse_Buf

  SELECT
   /* No-Value Keyword Processing */
   /* Global RECALLNONE and Storage Group RECALLNONE */
   WHEN (current_sg = '' & sclparm = structs.struct.10) |,
        (current_sg <> '' & sclparm = structs.struct.4) then
    do
     keyword = 0
   end
   /* Value Keyword Processing */
   /* Global FIRSTBACKUPGROUP */
   WHEN current_sg = '' & sclparm = structs.struct.1 then
    do
     keyword = 0

     /* Check if group exists */
     DO i = 1 to vscb.0
      if vscb.i = val then
       LEAVE
     END

     if i > vscb.0 then
      do
       pass_sg = val
       call Add_SG
     end

     if vscb.i.type = '' then
      vscb.i.type = 'B'

     if vscb.i.type <> 'B' then
      do
       msgid = "0336I"
       issue_msg.0 = 2
       issue_msg.1 = val
       issue_msg.2 = cbroam

       call Write_Str

       msgid = "ADD"
       issue_msg.0 = 1
       issue_msg.1 = "Storage group is either not object backup or was " ||,
                     "specified as another storage group type previously."

       call Write_Str
       QUEUE ' '

       err_found = 1
     end
     else
      do
       if vscb.i.bktype = 2 then
        do
         msgid = "0335I"
         issue_msg.0 = 2
         issue_msg.1 = val
         issue_msg.2 = cbroam

         call Write_Str
         QUEUE ' '

         err_found = 1
       end
       else
        do
         vscb.i.bktype = 1
         mcbvscbb = i
         mcbdobgp = 1
       end
     end
   end

   /* Global SECONDBACKUPGROUP */
   WHEN current_sg = '' & sclparm = structs.struct.2 then
    do
     keyword = 0
     mcbbk2on = 1

     /* Check if group exists */
     DO i = 1 to vscb.0
      if vscb.i = val then
       LEAVE
     END

     if i > vscb.0 then
      do
       pass_sg = val
       call Add_SG
     end

     if vscb.i.type = '' then
      vscb.i.type = 'B'

     if vscb.i.type <> 'B' then
      do
       msgid = "0336I"
       issue_msg.0 = 2
       issue_msg.1 = val
       issue_msg.2 = cbroam

       call Write_Str

       msgid = "ADD"
       issue_msg.0 = 1
       issue_msg.1 = "Storage group is either not object backup or was " ||,
                     "specified as another storage group type previously."

       call Write_Str
       QUEUE ' '

       err_found = 1
     end
     else
      do
       if vscb.i.bktype = 1 then
        do
         msgid = "0335I"
         issue_msg.0 = 2
         issue_msg.1 = val
         issue_msg.2 = cbroam

         call Write_Str
         QUEUE ' '

         err_found = 1
       end
       else
        do
         vscb.i.bktype = 2
         mcbvscb2 = i
       end
     end
   end

   /* Global CYCLEWINDOW */
   WHEN current_sg = '' & sclparm = structs.struct.3 then
    do
     keyword = 0

     if val <> 'STARTONLY' & val <> 'STARTSTOP' then
      do
       msgid = "0303I"
       issue_msg.0 = 3
       issue_msg.1 = sclparm
       issue_msg.2 = current_stmt
       issue_msg.3 = val

       call Write_Str

       msgid = "ADD"
       issue_msg.0 = 1
       issue_msg.1 = "Value should be 'STARTONLY' or 'STARTSTOP'."

       call Write_Str
       QUEUE ' '

       err_found = 1
     end
   end

   /* Global CLEAROLDLOC */
   WHEN current_sg = '' & sclparm = structs.struct.4 then
    do
     keyword = 0

     if val <> 'OPT' & val <> 'TAPE' & val <> 'BOTH' & val <> 'NONE' then
      do
       msgid = "0303I"
       issue_msg.0 = 3
       issue_msg.1 = sclparm
       issue_msg.2 = current_stmt
       issue_msg.3 = val

       call Write_Str

       msgid = "ADD"
       issue_msg.0 = 1
       issue_msg.1 = "Value should be 'OPT', 'TAPE', 'BOTH', or 'NONE'."

       call Write_Str
       QUEUE ' '

       err_found = 1
     end
   end

   /* Global MAXRECALLTASKS, MAXOSMCFAIL, RECALLCLOUD, RECALLOPTICAL,
      RECALLTAPE, and RECALLALL */
   WHEN current_sg = '' & (sclparm = structs.struct.5 |,
        sclparm = structs.struct.6 | sclparm = structs.struct.7 |,
        sclparm = structs.struct.8 | sclparm = structs.struct.9 |,
        sclparm = structs.struct.11) then
    do
     keyword = 0

     if val < 0 | val > 255 then
      do
       msgid = "0303I"
       issue_msg.0 = 3
       issue_msg.1 = sclparm
       issue_msg.2 = current_stmt
       issue_msg.3 = val

       call Write_Str

       msgid = "ADD"
       issue_msg.0 = 1
       issue_msg.1 = "Value should not be less than 0 or greater than 255."

       call Write_Str
       QUEUE ' '

       err_found = 1
     end
   end

   /* Global RECALLDISKSUBLEVEL */
   WHEN current_sg = '' & sclparm = structs.struct.12 then
    do
     keyword = 0

     if val <> 1 & val <> 2 then
      do
       msgid = "0303I"
       issue_msg.0 = 3
       issue_msg.1 = sclparm
       issue_msg.2 = current_stmt
       issue_msg.3 = val

       call Write_Str

       msgid = "ADD"
       issue_msg.0 = 1
       issue_msg.1 = "Value should be 1 or 2."

       call Write_Str
       QUEUE ' '

       err_found = 1
     end
   end

   /* Global BACKUPDELETE */
   WHEN current_sg = '' & sclparm = structs.struct.13 then
    do
     keyword = 0

     if val <> 'ON' & val <> 'OFF' then
      do
       msgid = "0303I"
       issue_msg.0 = 3
       issue_msg.1 = sclparm
       issue_msg.2 = current_stmt
       issue_msg.3 = val

       call Write_Str

       msgid = "ADD"
       issue_msg.0 = 1
       issue_msg.1 = "Value should be 'ON' or 'OFF'."

       call Write_Str
       QUEUE ' '

       err_found = 1
     end
   end

   /* STORAGEGROUP name */
   WHEN current_sg = '' & sclparm = structs.struct.14 then
    do
     keyword = 0
     current_sg = val
     sg_ltparens = 1
     ltparens = 0
     vscb_ptr = 0

     DO i = 1 to vscb.0
      if vscb.i = current_sg then
       do
        vscb_ptr = i
        LEAVE
      end
     END

     if vscb_ptr = 0 then
      do
       pass_sg = current_sg
       call Add_SG
       vscb_ptr = vscb.0
     end
   end

   /* Storage Group FIRSTBACKUPGROUP */
   WHEN current_sg <> '' & sclparm = structs.struct.1 then
    do
     keyword = 0
     DO i = 1 to vscb.0
      if vscb.i = val then
       do
        LEAVE
      end
     end

     /* If group is not backup or does not exist */
     if i > vscb.0 then
      do
       pass_sg = val
       call Add_SG
     end

     if vscb.i.type = '' then
      vscb.i.type = 'B'

     if vscb.i.type <> 'B' then
      do
       msgid = "0336I"
       issue_msg.0 = 2
       issue_msg.1 = val
       issue_msg.2 = cbroam

       call Write_Str

       msgid = "ADD"
       issue_msg.0 = 1
       issue_msg.1 = "Storage group is either not object backup or was " ||,
                     "specified as another storage group type previously."

       call Write_Str
       QUEUE ' '

       err_found = 1
     end
     else
      do
       /* Check if group is being used as second backup */
       if vscb.i.bktype = 2 then
        do
         msgid = "0335I"
         issue_msg.0 = 2
         issue_msg.1 = val
         issue_msg.2 = cbroam

         call Write_Str
         QUEUE ' '

         err_found = 1
       end
       else
        do
         vscb.i.bktype = 1
         if vscb_ptr <> 0 then
          do
           /* if current storage group is not object storage */
           if vscb.vscb_ptr.type = '' then
            vscb.vscb_ptr.type = 'G'

           if vscb.vscb_ptr.type <> 'G' then
            do
             msgid = "0344I"
             issue_msg.0 = 3
             issue_msg.1 = sclparm
             issue_msg.2 = vscb.vscb_ptr
             issue_msg.3 = current_stmt

             call Write_Str
             QUEUE ' '

             err_found = 1
           end
           else
            do
             /* Valid, set vars */
             vscb.vscb_ptr.1stbackup = i
           end
         end
       end
     end
   end

   /* Storage Group SECONDBACKUPGROUP */
   WHEN current_sg <> '' & sclparm = structs.struct.2 then
    do
     keyword = 0
     mcbbk2on = 1
     DO i = 1 to vscb.0
      if vscb.i = val then
       do
        LEAVE
      end
     end

     /* If group is not backup or does not exist */
     if i > vscb.0 then
      do
       pass_sg = val
       call Add_SG
     end

     if vscb.i.type = '' then
      vscb.i.type = 'B'

     if vscb.i.type <> 'B' then
      do
       msgid = "0336I"
       issue_msg.0 = 2
       issue_msg.1 = val
       issue_msg.2 = cbroam

       call Write_Str

       msgid = "ADD"
       issue_msg.0 = 1
       issue_msg.1 = "Storage group is either not object backup or was " ||,
                     "specified as another storage group type previously."

       call Write_Str
       QUEUE ' '

       err_found = 1
     end
     else
      do
       /* Check if group is being used as second backup */
       if vscb.i.bktype = 1 then
        do
         msgid = "0335I"
         issue_msg.0 = 2
         issue_msg.1 = val
         issue_msg.2 = cbroam

         call Write_Str
         QUEUE ' '

         err_found = 1
       end
       else
        do
         vscb.i.bktype = 2
         if vscb_ptr <> 0 then
          do
           /* if current storage group is not object storage */
           if vscb.vscb_ptr.type = '' then
            vscb.vscb_ptr.type = 'G'

           if vscb.vscb_ptr.type <> 'G' then
            do
             msgid = "0344I"
             issue_msg.0 = 3
             issue_msg.1 = sclparm
             issue_msg.2 = vscb.vscb_ptr
             issue_msg.3 = current_stmt

             call Write_Str
             QUEUE ' '

             err_found = 1
           end
           else
            do
             /* Valid, set vars */
             vscb.vscb_ptr.2ndbackup = i
           end
         end
       end
     end
   end

   /* Storage Group CLEAROLDLOC */
   WHEN current_sg <> '' & sclparm = structs.struct.3 then
    do
     keyword = 0

     if vscb_ptr <> 0 then
      do
       /* Check for valid obj sg */
       if val <> "OPT" & val <> "TAPE" & val <> "BOTH" & val <> "NONE" then
        do
         msgid = "0303I"
         issue_msg.0 = 3
         issue_msg.1 = sclparm
         issue_msg.2 = current_stmt
         issue_msg.3 = val

         call Write_Str

         msgid = "ADD"
         issue_msg.0 = 1
         issue_msg.1 = "Value should be 'OPT', 'TAPE', 'BOTH', or 'NONE'."

         call Write_Str
         QUEUE ' '

         err_found = 1
       end
     end
   end

   /* Storage Group RECALLCLOUD, RECALLOPTICAL, RECALLTAPE, and RECALLALL */
   WHEN current_sg <> '' & (sclparm = structs.struct.5 |,
        sclparm = structs.struct.6 | sclparm = structs.struct.7 |,
        sclparm = structs.struct.8) then
    do
     keyword = 0

     if val < 0 | val > 255 then
      do
       msgid = "0303I"
       issue_msg.0 = 3
       issue_msg.1 = sclparm
       issue_msg.2 = current_stmt
       issue_msg.3 = val

       call Write_Str

       msgid = "ADD"
       issue_msg.0 = 1
       issue_msg.1 = "Value should not be less than 0 or greater than 255."

       call Write_Str
       QUEUE ' '

       err_found = 1
     end
     else
      do
       if vscb_ptr <> 0 then
        do
         /* check for valid object storage group */
         if vscb.vscb_ptr.type = '' then
          vscb.vscb_ptr.type = 'G'

         if vscb.vscb_ptr.type <> 'G' then
          do
           msgid = "0344I"
           issue_msg.0 = 3
           issue_msg.1 = sclparm
           issue_msg.2 = vscb.vscb_ptr
           issue_msg.3 = current_stmt

           call Write_Str
           QUEUE ' '

           err_found = 1
         end
       end
     end
   end

   /* Storage Group RECALLOFF */
   WHEN current_sg <> '' & sclparm = structs.struct.9 then
    do
     keyword = 0

     if vscb_ptr <> 0 then
      do
       /* check for valid object storage group */
       if vscb.vscb_ptr.type = '' then
        vscb.vscb_ptr.type = 'G'

       if vscb.vscb_ptr.type = 'G' then
        do
         if val <> "ON" & val <> "OFF" then
          do
           msgid = "0303I"
           issue_msg.0 = 3
           issue_msg.1 = sclparm
           issue_msg.2 = current_stmt
           issue_msg.3 = val

           call Write_Str

           msgid = "ADD"
           issue_msg.0 = 1
           issue_msg.1 = "Value should be 'ON' or 'OFF'."

           call Write_Str
           QUEUE ' '

           err_found = 1
         end
       end
       else
        do
         msgid = "0344I"
         issue_msg.0 = 3
         issue_msg.1 = sclparm
         issue_msg.2 = vscb.vscb_ptr
         issue_msg.3 = current_stmt

         call Write_Str
         QUEUE ' '

         err_found = 1
       end
     end
   end

   /* Storage Group RECALLDISKSUBLEVEL */
   WHEN current_sg <> '' & sclparm = structs.struct.10 then
    do
     keyword = 0

     if val <> 1 & val <> 2 then
      do
       msgid = "0303I"
       issue_msg.0 = 3
       issue_msg.1 = sclparm
       issue_msg.2 = current_stmt
       issue_msg.3 = val

       call Write_Str

       msgid = "ADD"
       issue_msg.0 = 1
       issue_msg.1 = "Value should be 1 or 2."

       call Write_Str
       QUEUE ' '

       err_found = 1
     end
     else
      do
       if vscb_ptr <> 0 then
        do
         /* check for valid object storage group */
         if vscb.vscb_ptr.type = '' then
          vscb.vscb_ptr.type = 'G'

         if vscb.vscb_ptr.type <> 'G' then
          do
           msgid = "0344I"
           issue_msg.0 = 3
           issue_msg.1 = sclparm
           issue_msg.2 = vscb.vscb_ptr
           issue_msg.3 = current_stmt

           call Write_Str
           QUEUE ' '

           err_found = 1
         end
       end
     end
   end

   /* Storage Group BACKUPTIER */
   WHEN current_sg <> '' & sclparm = structs.struct.11 then
    do
     keyword = 0

     if vscb_ptr <> 0 then
      do
       /* check for valid backup storage group */
       if vscb.vscb_ptr.type = '' then
        vscb.vscb_ptr.type = 'B'

       if vscb.vscb_ptr.type = 'B' then
        do
         if val <> "TAPEOPT" & val <> "FS" & val <> "CLOUD" then
          do
           msgid = "0303I"
           issue_msg.0 = 3
           issue_msg.1 = sclparm
           issue_msg.2 = current_stmt
           issue_msg.3 = val

           call Write_Str

           msgid = "ADD"
           issue_msg.0 = 1
           issue_msg.1 = "Value should be 'TAPEOPT', 'FS', or 'CLOUD'."

           call Write_Str
           QUEUE ' '

           err_found = 1
         end
       end
       else
        do
         msgid = "0344I"
         issue_msg.0 = 3
         issue_msg.1 = sclparm
         issue_msg.2 = vscb.vscb_ptr
         issue_msg.3 = current_stmt

         call Write_Str
         QUEUE ' '

         err_found = 1
       end
     end
   end

   /* Ending parenthesis for STORAGEGROUP parameter */
   WHEN sclparm = "SG_RTPARENS" then
    do
     sg_ltparens = 0
     current_sg = ''
     vscb_ptr = 0
     buf = STRIP(SUBSTR(buf,2))
   end

   /* ENDING CHARACTER NOT A ')' (I.E. - EMBEDDED BLANK OR COMMA) */
   WHEN sclparm = "INV_END" then
    do
     keyword = 0
     ltparens = 0
     brk = POS(")",buf)
     if brk = 0 then
      brk = LENGTH(buf)
     extra = STRIP(SUBSTR(buf,1,brk))
     buf = STRIP(SUBSTR(buf,brk + 1))

     if LENGTH(extra) > 1 then
      do
       msgid = "0304I"
       issue_msg.0 = 3
       issue_msg.1 = structs.struct.scluindx
       issue_msg.2 = current_stmt
       issue_msg.3 = extra

       call Write_Str
       QUEUE ' '

       err_found = 1
     end
   end

   /*  KEYWORD VALUE SYNTAX INCORRECT                                  */
   /*                                                                  */
   /*  BAD SYNTAX CAN BE ONE OF THE FOLLOWING:                         */
   /*                                                                  */
   /*  - THE LENGTH OF THE DATA IS WRONG.  FOR EXAMPLE, THE LENGTH     */
   /*    SHOULD BE BETWEEN 1 AND 3, AND THE CUSTOMER HAS ENTERED DATA  */
   /*    WITH A LENGTH OF 4.                                           */
   /*  - THE VALUE IS SUPPOSED TO BE NUMERIC, AND THERE IS AN          */
   /*    ALPHABETIC CHARACTER IN IT.                                   */
   /*  - THE DATA IS SUPPOSED TO START WITH A LETTER, AND IT STARTS    */
   /*    WITH A NUMBER                                                 */
   /*  - THE DATA IS SUPPOSED TO ONLY BE LETTERS AND NUMBERS, AND IT   */
   /*    HAS A SPECIAL CHARACTER IN IT                                 */
   WHEN sclparm = "SYNTAX_ERR" then
    do
     keyword = 0
     err_found = 1
   end

   /* Unknown keyword or keyword starting with left parens */
   WHEN sclparm = "INV_KEYWD" then
    do
     /* Unknown Keyword */
     if POS('(',buf) = 1 then
      do
       msgid = "0302I"
       issue_msg.0 = 2
       issue_msg.1 = '('
       issue_msg.2 = current_stmt

       call Write_Str
       QUEUE ' '

       buf = SUBSTR(buf,2)
     end
     else
      do
       brk = LENGTH(buf) + 1
       sp = POS(" ",buf)
       cm = POS(",",buf)
       rp = POS(")",buf)
       if sp <> 0 then
        brk = sp
       if cm <> 0 & cm < brk then
        brk = cm
       if rp <> 0 & rp < brk then
        do
         ltparens = 0
         brk = rp
       end
       brk = brk + 1

       extra = STRIP(SUBSTR(buf,1,brk - 1))
       buf = STRIP(SUBSTR(buf,brk))
       msgid = "0302I"
       issue_msg.0 = 2
       issue_msg.1 = extra
       issue_msg.2 = current_stmt

       call Write_Str
       QUEUE ' '
     end
     err_found = 1
   end
   OTHERWISE
     say "Unknown Error "sclparm" in SETOSMC SG '"current_sg"'"
  END /* DO WHILE buf <> '' */
  keyword = 0
 END

 /* Last keyword was missing end parenthesis */
 if ltparens <> 0 then
  do
   msgid = "0303I"
   issue_msg.0 = 3
   issue_msg.1 = sclparm
   issue_msg.2 = current_stmt
   issue_msg.3 = val

   call Write_Str

   msgid = "ADD"
   issue_msg.0 = 1
   issue_msg.1 = "Missing end parenthesis."

   call Write_Str
   QUEUE ' '

   ltparens = 0
   err_found = 1
 end

 /* Last STORAGEGROUP missing end parenthesis */
 if sg_ltparens <> 0 then
  do
   msgid = "0305I"
   issue_msg.0 = 2
   issue_msg.1 = current_sg
   issue_msg.2 = current_stmt

   call Write_Str
   QUEUE ' '

   sg_ltparens = 0
   err_found = 1
   current_sg = ''
   vscb_ptr = 0
 end

 /* Last keyword missing data */
 if keyword <> 0 then
  do
   msgid = "0306I"
   issue_msg.0 = 2
   issue_msg.1 = sclparm
   issue_msg.2 = current_stmt

   call Write_Str
   QUEUE ' '

   err_found = 1
 end
 return

/*-------------------------------------------------------------------*/
/* Routine: Validate_SETDISK                                         */
/* Purpose: Validate the SETDISK statement.                          */
/*          Loops throught the buffer of keywords and values, using  */
/*          Parse_Buf to extract keyword/value pairs. Check the      */
/*          values and issue error messages for invalid values given.*/
/*-------------------------------------------------------------------*/
Validate_SETDISK:

 call SETDISK_STRUCT
 err_found = 0

 DO WHILE buf <> ''
  call Parse_Buf

  SELECT
   /* STORAGEGROUP Name */
   WHEN current_sg = '' & sclparm = structs.struct.1 then
    do
     keyword = 0
     current_sg = val
     sg_ltparens = 1
     ltparens = 0
     vscb_ptr = 0

     DO i = 1 to vscb.0
      if vscb.i = current_sg then
       do
        vscb_ptr = i
        LEAVE
      end
     END
     if vscb_ptr = 0 then
      do
       pass_sg = current_sg
       call Add_SG
       vscb_ptr = vscb.0
     end

     vscb.vscb_ptr.setdisk = 1
   end

   /* Storage Group L2TYPE */
   WHEN current_sg <> '' & sclparm = structs.struct.1 then
    do
     keyword = 0
     if val <> 'ZFS' & val <> 'NFS' then
      do
       msgid = "0303I"
       issue_msg.0 = 3
       issue_msg.1 = sclparm
       issue_msg.2 = current_stmt
       issue_msg.3 = val

       call Write_Str

       msgid = "ADD"
       issue_msg.0 = 1
       issue_msg.1 = "Value should be 'ZFS' or 'NFS'."

       call Write_Str
       QUEUE ' '

       err_found = 1
     end
     else
      do
       if val = 'ZFS' then
        do
         vscb.vscb_ptr.vscdstat_l2t_zfs = 1
         vscb.vscb_ptr.vscdstat_l2t_nfs = 0
       end
       else
        do
         vscb.vscb_ptr.vscdstat_l2t_zfs = 0
         vscb.vscb_ptr.vscdstat_l2t_nfs = 1
       end
     end
   end

   /* Storage Group L2DIR */
   WHEN current_sg <> '' & sclparm = structs.struct.2 then
    do
     keyword = 0
     vscb.vscb_ptr.vscdl2dir = val
   end

   /* Ending parenthesis for STORAGEGROUP parameter */
   WHEN sclparm = "SG_RTPARENS" then
    do
     sg_ltparens = 0
     current_sg = ''
     vscb_ptr = 0
     buf = STRIP(SUBSTR(buf,2))
   end

   /* ENDING CHARACTER NOT A ')' (I.E. - EMBEDDED BLANK OR COMMA) */
   WHEN sclparm = "INV_END" then
    do
     keyword = 0
     ltparens = 0
     brk = POS(")",buf)
     if brk = 0 then
      brk = LENGTH(buf)
     extra = STRIP(SUBSTR(buf,1,brk))
     buf = STRIP(SUBSTR(buf,brk + 1))

     if LENGTH(extra) > 1 then
      do
       msgid = "0304I"
       issue_msg.0 = 3
       issue_msg.1 = structs.struct.scluindx
       issue_msg.2 = current_stmt
       issue_msg.3 = extra

       call Write_Str
       QUEUE ' '

       err_found = 1
     end
   end

   /*  KEYWORD VALUE SYNTAX INCORRECT                                  */
   /*                                                                  */
   /*  BAD SYNTAX CAN BE ONE OF THE FOLLOWING:                         */
   /*                                                                  */
   /*  - THE LENGTH OF THE DATA IS WRONG.  FOR EXAMPLE, THE LENGTH     */
   /*    SHOULD BE BETWEEN 1 AND 3, AND THE CUSTOMER HAS ENTERED DATA  */
   /*    WITH A LENGTH OF 4.                                           */
   /*  - THE VALUE IS SUPPOSED TO BE NUMERIC, AND THERE IS AN          */
   /*    ALPHABETIC CHARACTER IN IT.                                   */
   /*  - THE DATA IS SUPPOSED TO START WITH A LETTER, AND IT STARTS    */
   /*    WITH A NUMBER                                                 */
   /*  - THE DATA IS SUPPOSED TO ONLY BE LETTERS AND NUMBERS, AND IT   */
   /*    HAS A SPECIAL CHARACTER IN IT                                 */
   WHEN sclparm = "SYNTAX_ERR" then
    do
     keyword = 0
     err_found = 1
   end

   /* Unknown keyword or keyword starting with left parens */
   WHEN sclparm = "INV_KEYWD" then
    do
     /* Unknown Keyword */
     if POS('(',buf) = 1 then
      do
       msgid = "0302I"
       issue_msg.0 = 2
       issue_msg.1 = '('
       issue_msg.2 = current_stmt

       call Write_Str
       QUEUE ' '

       buf = SUBSTR(buf,2)
     end
     else
      do
       brk = LENGTH(buf) + 1
       sp = POS(" ",buf)
       cm = POS(",",buf)
       rp = POS(")",buf)
       if sp <> 0 then
        brk = sp
       if cm <> 0 & cm < brk then
        brk = cm
       if rp <> 0 & rp < brk then
        do
         ltparens = 0
         brk = rp
       end
       brk = brk + 1

       extra = STRIP(SUBSTR(buf,1,brk - 1))
       buf = STRIP(SUBSTR(buf,brk))

       msgid = "0302I"
       issue_msg.0 = 0
       issue_msg.1 = extra
       issue_msg.2 = current_stmt

       call Write_Str
       QUEUE ' '
     end
     err_found = 1
   end
   OTHERWISE
    say "Unknown Error "sclparm" in SETDISK SG "current_sg
   end
  keyword = 0
 END /* DO WHILE buf <> '' */

 /* Last keyword was missing end parenthesis */
 if ltparens <> 0 then
  do
   msgid = "0303I"
   issue_msg.0 = 3
   issue_msg.1 = sclparm
   issue_msg.2 = current_stmt
   issue_msg.3 = val

   call Write_Str

   msgid = "ADD"
   issue_msg.0 = 1
   issue_msg.1 = "Missing end parenthesis."

   call Write_Str
   QUEUE ' '

   ltparens = 0
   err_found = 1
 end

 /* Last STORAGEGROUP missing end parenthesis */
 if sg_ltparens <> 0 then
  do
   msgid = "0305I"
   issue_msg.0 = 2
   issue_msg.1 = current_sg
   issue_msg.2 = current_stmt

   call Write_Str
   QUEUE ' '

   sg_ltparens = 0
   err_found = 1
   current_sg = ''
   vscb_ptr = 0
 end

 /* Last keyword missing data */
 if keyword <> 0 then
  do
   msgid = "0306I"
   issue_msg.0 = 2
   issue_msg.1 = sclparm
   issue_msg.2 = current_stmt

   call Write_Str
   QUEUE ' '

   err_found = 1
 end
 return

/*-------------------------------------------------------------------*/
/* Routine: Validate_SETTLIB                                         */
/* Purpose: Validate the SETTLIB statement.                          */
/*          Loops throught the buffer of keywords and values, using  */
/*          Parse_Buf to extract keyword/value pairs. Check the      */
/*          values and issue error messages for invalid values given.*/
/*-------------------------------------------------------------------*/
Validate_SETTLIB:

 call SETTLIB_STRUCT
 err_found = 0

 DO WHILE buf <> ''
  call Parse_Buf
  SELECT
   /* Global LACSRETRYFAIL */
   WHEN sclparm = structs.struct.1 then
    do
     keyword = 0
     if val <> 'ON' & val <> 'OFF' then
      do
       msgid = "0303I"
       issue_msg.0 = 3
       issue_msg.1 = sclparm
       issue_msg.2 = current_stmt
       issue_msg.3 = val

       call Write_Str

       msgid = "ADD"
       issue_msg.0 = 1
       issue_msg.1 = "Value should be 'ON' or 'OFF'."

       call Write_Str
       QUEUE ' '

       err_found = 1
     end
   end

   /* Global LACSRETRYMINUTES */
   WHEN sclparm = structs.struct.2 then
    do
     keyword = 0
     if val <= 0 then
      do
       msgid = "WARN1"
       issue_msg.0 = 4
       issue_msg.1 = sclparm
       issue_msg.2 = current_stmt
       issue_msg.3 = "<= 0"
       issue_msg.4 = "2"

       call Write_Str
       QUEUE ' '

       warn_found = 1
     end
     ovtlarem = 1
   end

   /* Global LACSRETRYTIMES */
   WHEN sclparm = structs.struct.3 then
    do
     keyword = 0
     if val <= 0 then
      do
       msgid = "WARN1"
       issue_msg.0 = 4
       issue_msg.1 = sclparm
       issue_msg.2 = current_stmt
       issue_msg.3 = "<= 0"
       issue_msg.4 = "6"

       call Write_Str
       QUEUE ' '

       warn_found = 1
     end
     ovtlarts = 1
   end

   /* Global SCRATCHTHRESHOLDMSG */
   WHEN sclparm = structs.struct.4 then
    do
     keyword = 0
     if val <= 0 | val > 100 then
      do
       msgid = "WARN1"
       issue_msg.0 = 4
       issue_msg.1 = sclparm
       issue_msg.2 = current_stmt
       issue_msg.3 = "<= 0 or > 100"
       issue_msg.4 = "100"

       call Write_Str
       QUEUE ' '

       warn_found = 1
     end
     else if val < 25 then
      do
       msgid = "WARN1"
       issue_msg.0 = 4
       issue_msg.1 = sclparm
       issue_msg.2 = current_stmt
       issue_msg.3 = "< 25"
       issue_msg.4 = "25"

       call Write_Str
       QUEUE ' '

       warn_found = 1
     end
   end

   /* Global ENTRYIGNOREMSGTYPE */
   WHEN sclparm = structs.struct.5 then
    do
     keyword = 0
     if val <> "DETAIL" & val <> "SUMMARY" & val <> "SUPPRESS" then
      do
       msgid = "0303I"
       issue_msg.0 = 3
       issue_msg.1 = sclparm
       issue_msg.2 = current_stmt
       issue_msg.3 = val

       call Write_Str

       msgid = "ADD"
       issue_msg.0 = 1
       issue_msg.1 = "Value should be 'DETAIL' 'SUMMARY' or 'SUPPRESS'."

       call Write_Str
       QUEUE ' '

       err_found = 1
     end
   end

   /* Global ENTRYIGNOREMSGDEST */
   WHEN sclparm = structs.struct.6 then
    do
     keyword = 0
     if val <> 'ALL' & val <> 'SYSLOG' then
      do
       msgid = "0303I"
       issue_msg.0 = 3
       issue_msg.1 = sclparm
       issue_msg.2 = current_stmt
       issue_msg.3 = val

       call Write_Str

       msgid = "ADD"
       issue_msg.0 = 1
       issue_msg.1 = "Value should be 'ALL' or 'SYSLOG'."

       call Write_Str
       QUEUE ' '

       err_found = 1
     end
   end

   /* Global ENTRYSUCCESSMSGDEST */
   WHEN sclparm = structs.struct.7 then
    do
     keyword = 0
     if val <> 'ALL' & val <> 'SYSLOG' then
      do
       msgid = "0303I"
       issue_msg.0 = 3
       issue_msg.1 = sclparm
       issue_msg.2 = current_stmt
       issue_msg.3 = val

       call Write_Str

       msgid = "ADD"
       issue_msg.0 = 1
       issue_msg.1 = "Value should be 'ALL' or 'SYSLOG'."

       call Write_Str
       QUEUE ' '

       err_found = 1
     end
   end

   /* ENDING CHARACTER NOT A ')' (I.E. - EMBEDDED BLANK OR COMMA) */
   WHEN sclparm = "INV_END" then
    do
     keyword = 0
     ltparens = 0
     brk = POS(")",buf)
     if brk = 0 then
      brk = LENGTH(buf)
     extra = STRIP(SUBSTR(buf,1,brk))
     buf = STRIP(SUBSTR(buf,brk + 1))

     if LENGTH(extra) > 1 then
      do
       msgid = "0304I"
       issue_msg.0 = 3
       issue_msg.1 = structs.struct.scluindx
       issue_msg.2 = current_stmt
       issue_msg.3 = extra

       call Write_Str
       QUEUE ' '

       err_found = 1
     end
   end

   /*  KEYWORD VALUE SYNTAX INCORRECT                                  */
   /*                                                                  */
   /*  BAD SYNTAX CAN BE ONE OF THE FOLLOWING:                         */
   /*                                                                  */
   /*  - THE LENGTH OF THE DATA IS WRONG.  FOR EXAMPLE, THE LENGTH     */
   /*    SHOULD BE BETWEEN 1 AND 3, AND THE CUSTOMER HAS ENTERED DATA  */
   /*    WITH A LENGTH OF 4.                                           */
   /*  - THE VALUE IS SUPPOSED TO BE NUMERIC, AND THERE IS AN          */
   /*    ALPHABETIC CHARACTER IN IT.                                   */
   /*  - THE DATA IS SUPPOSED TO START WITH A LETTER, AND IT STARTS    */
   /*    WITH A NUMBER                                                 */
   /*  - THE DATA IS SUPPOSED TO ONLY BE LETTERS AND NUMBERS, AND IT   */
   /*    HAS A SPECIAL CHARACTER IN IT                                 */
   WHEN sclparm = "SYNTAX_ERR" then
    do
     keyword = 0
     err_found = 1
   end

   /* Unknown keyword or keyword starting with left parens */
   WHEN sclparm = "INV_KEYWD" then
    do
     /* Unknown Keyword */
     if POS('(',buf) = 1 then
      do
       msgid = "0302I"
       issue_msg.0 = 2
       issue_msg.1 = '('
       issue_msg.2 = current_stmt

       call Write_Str
       QUEUE ' '

       buf = SUBSTR(buf,2)
     end
     else
      do
       brk = LENGTH(buf) + 1
       sp = POS(" ",buf)
       cm = POS(",",buf)
       rp = POS(")",buf)
       if sp <> 0 then
        brk = sp
       if cm <> 0 & cm < brk then
        brk = cm
       if rp <> 0 & rp < brk then
        do
         ltparens = 0
         brk = rp
       end
       brk = brk + 1

       extra = STRIP(SUBSTR(buf,1,brk - 1))
       buf = STRIP(SUBSTR(buf,brk))

       msgid = "0302I"
       issue_msg.0 = 2
       issue_msg.1 = extra
       issue_msg.2 = current_stmt

       call Write_Str
       QUEUE ' '
     end
     err_found = 1
   end
   OTHERWISE
    say "Unknown Error "sclparm
  end
  keyword = 0
 END /* DO WHILE buf <> '' */

 /* Last keyword was missing end parenthesis */
 if ltparens <> 0 then
  do
   msgid = "0303I"
   issue_msg.0 = 3
   issue_msg.1 = sclparm
   issue_msg.2 = current_stmt
   issue_msg.3 = val

   call Write_Str

   msgid = "ADD"
   issue_msg.0 = 1
   issue_msg.1 = "Missing end parenthesis."

   call Write_Str
   QUEUE ' '

   ltparens = 0
   err_found = 1
 end

 /* Last keyword missing data */
 if keyword <> 0 then
  do
   msgid = "0306I"
   issue_msg.0 = 2
   issue_msg.1 = sclparm
   issue_msg.2 = current_stmt

   call Write_Str
   QUEUE ' '

   err_found = 1
 end
 return

/*-------------------------------------------------------------------*/
/* Routine: Validate_SETCLOUD                                        */
/* Purpose: Validate the SETCLOUD statement.                         */
/*          Loops throught the buffer of keywords and values, using  */
/*          Parse_Buf to extract keyword/value pairs. Check the      */
/*          values and issue error messages for invalid values given.*/
/*-------------------------------------------------------------------*/
Validate_SETCLOUD:

 call SETCLOUD_STRUCT
 err_found = 0
 cont_validchars = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstu' ||,
                   'vwxyz0123456789-.'

 DO WHILE buf <> ''
  call Parse_Buf

  SELECT
   /* Global PROVIDER and Storage Group PROVIDER */
   WHEN sclparm = structs.struct.1 then
    do
     keyword = 0
     if current_sg = '' then
      mcbcprov = val
     if vscb_ptr <> 0 then
      vscb.vscb_ptr.vsccprov = val
   end

   /* Global CONTAINER and Storage Group CONTAINER */
   WHEN sclparm = structs.struct.2 then
    do
     keyword = 0
     if VERIFY(val,cont_validchars) then
      do
       msgid = "0303I"
       issue_msg.0 = 3
       issue_msg.1 = sclparm
       issue_msg.2 = current_stmt
       issue_msg.3 = val

       call Write_Str

       msgid = "ADD"
       issue_msg.0 = 1
       issue_msg.1 = "Value should contain only uppercase/lowercase " ||,
                     "characters numbers, and the characters '-.'."

       call Write_Str
       QUEUE ' '

       err_found = 1
     end
     else
      do
       if current_sg = '' then
        mcbccont = val
       if vscb_ptr <> 0 then
        vscb.vscb_ptr.vscccont = val
     end
   end

   /* STORAGEGROUP Name */
   WHEN current_sg = '' & sclparm = structs.struct.3 then
    do
     keyword = 0
     current_sg = val
     sg_ltparens = 1
     ltparens = 0
     vscb_ptr = 0

     DO i = 1 to vscb.0
      if vscb.i = current_sg then
       do
        vscb_ptr = i
        LEAVE
      end
     END
     if vscb_ptr = 0 then
      do
       pass_sg = current_sg
       call Add_SG
       vscb_ptr = vscb.0
     end
   end

   /* Ending parenthesis for STORAGEGROUP parameter */
   WHEN sclparm = "SG_RTPARENS" then
    do
     sg_ltparens = 0
     current_sg = ''
     vscb_ptr = 0
     buf = STRIP(SUBSTR(buf,2))
   end

   /* ENDING CHARACTER NOT A ')' (I.E. - EMBEDDED BLANK OR COMMA) */
   WHEN sclparm = "INV_END" then
    do
     keyword = 0
     ltparens = 0
     brk = POS(")",buf)
     if brk = 0 then
      brk = LENGTH(buf)
     extra = STRIP(SUBSTR(buf,1,brk))
     buf = STRIP(SUBSTR(buf,brk + 1))

     if LENGTH(extra) > 1 then
      do
       msgid = "0304I"
       issue_msg.0 = 3
       issue_msg.1 = structs.struct.scluindx
       issue_msg.2 = current_stmt
       issue_msg.3 = extra

       call Write_Str
       QUEUE ' '

       err_found = 1
     end
   end

   /*  KEYWORD VALUE SYNTAX INCORRECT                                  */
   /*                                                                  */
   /*  BAD SYNTAX CAN BE ONE OF THE FOLLOWING:                         */
   /*                                                                  */
   /*  - THE LENGTH OF THE DATA IS WRONG.  FOR EXAMPLE, THE LENGTH     */
   /*    SHOULD BE BETWEEN 1 AND 3, AND THE CUSTOMER HAS ENTERED DATA  */
   /*    WITH A LENGTH OF 4.                                           */
   /*  - THE VALUE IS SUPPOSED TO BE NUMERIC, AND THERE IS AN          */
   /*    ALPHABETIC CHARACTER IN IT.                                   */
   /*  - THE DATA IS SUPPOSED TO START WITH A LETTER, AND IT STARTS    */
   /*    WITH A NUMBER                                                 */
   /*  - THE DATA IS SUPPOSED TO ONLY BE LETTERS AND NUMBERS, AND IT   */
   /*    HAS A SPECIAL CHARACTER IN IT                                 */
   WHEN sclparm = "SYNTAX_ERR" then
    do
     keyword = 0
     err_found = 1
   end

   /* Unknown keyword or keyword starting with left parens */
   WHEN sclparm = "INV_KEYWD" then
    do
     /* Unknown Keyword */
     if POS('(',buf) = 1 then
      do
       msgid = "0302I"
       issue_msg.0 = 2
       issue_msg.1 = '('
       issue_msg.2 = current_stmt

       call Write_Str
       QUEUE ' '

       buf = SUBSTR(buf,2)
     end
     else
      do
       brk = LENGTH(buf) + 1
       sp = POS(" ",buf)
       cm = POS(",",buf)
       rp = POS(")",buf)
       if sp <> 0 then
        brk = sp
       if cm <> 0 & cm < brk then
        brk = cm
       if rp <> 0 & rp < brk then
        do
         ltparens = 0
         brk = rp
       end
       brk = brk + 1

       extra = STRIP(SUBSTR(buf,1,brk - 1))
       buf = STRIP(SUBSTR(buf,brk))

       msgid = "0302I"
       issue_msg.0 = 2
       issue_msg.1 = extra
       issue_msg.2 = current_stmt

       call Write_Str
       QUEUE ' '
     end
     err_found = 1
   end
   OTHERWISE
    say "Unknown Error '"sclparm"' in SETCLOUD SG "current_sg
  END /* SELECT */
  keyword = 0
 END /* DO WHILE buf <> '' */

 /* Last keyword was missing end parenthesis */
 if ltparens <> 0 then
  do
   msgid = "0303I"
   issue_msg.0 = 3
   issue_msg.1 = sclparm
   issue_msg.2 = current_stmt
   issue_msg.3 = val

   call Write_Str

   msgid = "ADD"
   issue_msg.0 = 1
   issue_msg.1 = "Missing end parenthesis."

   call Write_Str
   QUEUE ' '

   ltparens = 0
   err_found = 1
 end

 /* Last STORAGEGROUP missing end parenthesis */
 if sg_ltparens <> 0 then
  do
   msgid = "0305I"
   issue_msg.0 = 2
   issue_msg.1 = current_sg
   issue_msg.2 = current_stmt

   call Write_Str
   QUEUE ' '

   sg_ltparens = 0
   err_found = 1
   current_sg = ''
   vscb_ptr = 0
 end

 /* Last keyword missing data */
 if keyword <> 0 then
  do
   msgid = "0306I"
   issue_msg.0 = 2
   issue_msg.1 = sclparm
   issue_msg.2 = current_stmt

   call Write_Str
   QUEUE ' '

   err_found = 1
 end
 return

/*-------------------------------------------------------------------*/
/* Routine: Validate_ONLYIF                                          */
/* Purpose: Validates the ONLYIF statement.                          */
/*          Loops throught the buffer of keywords and values, using  */
/*          Parse_Buf to extract keyword/value pairs. Check the      */
/*          values and issue error messages for invalid values given.*/
/*          If cross checking is enabled, parms will be checked      */
/*          against user defined parms to determine if set statement */
/*          processing should occur.                                 */
/*-------------------------------------------------------------------*/
Validate_ONLYIF:
 onlyif_validchars = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstu' ||,
                     'vwxyz0123456789*'
 err_found = 0

 call ONLYIF_STRUCT

 /* If doing crosschecking, reset onlyif control vars */
 if ex_onlyif = 1 then
  do
   set_sysname = 0
   set_db2id = 0
   mcbppivr = 0
   mcbppign = 1
 end

 DO WHILE buf <> ''
  call Parse_Buf

  SELECT
   /* Global SYSNAME */
   WHEN sclparm = structs.struct.1 then
    do
     keyword = 0

     if VERIFY(val,onlyif_validchars) <> 0 then
      do
       msgid = "0303I"
       issue_msg.0 = 3
       issue_msg.1 = sclparm
       issue_msg.2 = current_stmt
       issue_msg.3 = val

       call Write_Str

       msgid = "ADD"
       issue_msg.0 = 1
       issue_msg.1 = "Value should contain only uppercase/lowercase " ||,
                     "characters numbers, and the character '*'."

       call Write_Str
       QUEUE ' '

       err_found = 1
       set_sysname = 1
     end
     else
      do
       /* determine if processing */
       if ex_onlyif = 1 then
        do
         if val = mcbsysnm | val = "*ALL*" then
          set_sysname = 0
         else
          set_sysname = 1

         if set_sysname = 0 & set_db2id = 0 & mcbppivr = 0 then
          mcbppign = 1
         else
          mcbppign = 0
       end
     end
   end

   /* Global DB2ID */
   WHEN sclparm = structs.struct.2 then
    do
     keyword = 0

     if VERIFY(val,onlyif_validchars) <> 0 then
      do
       msgid = "0303I"
       issue_msg.0 = 3
       issue_msg.1 = sclparm
       issue_msg.2 = current_stmt
       issue_msg.3 = val

       call Write_Str

       msgid = "ADD"
       issue_msg.0 = 1
       issue_msg.1 = "Value should contain only uppercase/lowercase " ||,
                     "characters numbers, and the character '*'."

       call Write_Str
       QUEUE ' '

       err_found = 1
     end
     else
      do
       if LENGTH(val) > 4 & val <> "*ALL*" then
        do
         msgid = "0303I"
         issue_msg.0 = 3
         issue_msg.1 = sclparm
         issue_msg.2 = current_stmt
         issue_msg.3 = val

         call Write_Str

         msgid = "ADD"
         issue_msg.0 = 1
         issue_msg.1 = "Value should be no longer than 4 characters or " ||,
                       "be '*ALL*."

         call Write_Str
         QUEUE ' '

         err_found = 1
         set_db2id = 1
       end
       else
        do
         /* determine if processing */
         if ex_onlyif = 1 then
          do
           if val = db2ssid | val = "*ALL*" then
            set_db2id = 0
           else
            set_db2id = 1

           if set_sysname = 0 & set_db2id = 0 & mcbppivr = 0 then
            mcbppign = 1
           else
            mcbppign = 0
         end
       end
     end
   end

   /* Global OAMVAR */
   WHEN sclparm = structs.struct.3 then
    do
     keyword = 0

     if VERIFY(val,onlyif_validchars) <> 0 then
      do
       msgid = "0303I"
       issue_msg.0 = 3
       issue_msg.1 = sclparm
       issue_msg.2 = current_stmt
       issue_msg.3 = val

       call Write_Str

       msgid = "ADD"
       issue_msg.0 = 1
       issue_msg.1 = "Value should contain only uppercase/lowercase " ||,
                     "characters numbers, and the character '*'."

       call Write_Str
       QUEUE ' '

       err_found = 1
       mcbppivr = 1
     end
     else
      do
       /* determine if processing */
       if ex_onlyif = 1 then
        do
         if val = oamvar then
          mcbppivr = 0
         else
          mcbppivr = 1

         if set_sysname = 0 & set_db2id = 0 & mcbppivr = 0 then
          mcbppign = 1
         else
          mcbppign = 0
       end
     end
   end

   /* ENDING CHARACTER NOT A ')' (I.E. - EMBEDDED BLANK OR COMMA) */
   WHEN sclparm = "INV_END" then
    do
     keyword = 0
     ltparens = 0
     brk = POS(")",buf)
     if brk = 0 then
      brk = LENGTH(buf)
     extra = STRIP(SUBSTR(buf,1,brk))
     buf = STRIP(SUBSTR(buf,brk + 1))
     if LENGTH(extra) > 1 then
      do
       msgid = "0304I"
       issue_msg.0 = 3
       issue_msg.1 = structs.struct.scluindx
       issue_msg.2 = current_stmt
       issur_msg.3 = extra

       call Write_Str
       QUEUE ' '

       err_found = 1
     end
   end

   /*  KEYWORD VALUE SYNTAX INCORRECT                                  */
   /*                                                                  */
   /*  BAD SYNTAX CAN BE ONE OF THE FOLLOWING:                         */
   /*                                                                  */
   /*  - THE LENGTH OF THE DATA IS WRONG.  FOR EXAMPLE, THE LENGTH     */
   /*    SHOULD BE BETWEEN 1 AND 3, AND THE CUSTOMER HAS ENTERED DATA  */
   /*    WITH A LENGTH OF 4.                                           */
   /*  - THE VALUE IS SUPPOSED TO BE NUMERIC, AND THERE IS AN          */
   /*    ALPHABETIC CHARACTER IN IT.                                   */
   /*  - THE DATA IS SUPPOSED TO START WITH A LETTER, AND IT STARTS    */
   /*    WITH A NUMBER                                                 */
   /*  - THE DATA IS SUPPOSED TO ONLY BE LETTERS AND NUMBERS, AND IT   */
   /*    HAS A SPECIAL CHARACTER IN IT                                 */
   WHEN sclparm = "SYNTAX_ERR" then
    do
     keyword = 0
     err_found = 1
   end

   /* Unknown keyword or keyword starting with left parens */
   WHEN sclparm = "INV_KEYWD" then
    do
     /* Unknown Keyword */
     if POS('(',buf) = 1 then
      do
       msgid = "0302I"
       issue_msg.0 = 2
       issue_msg.1 = '('
       issue_msg.2 = current_stmt

       call Write_Str
       QUEUE ' '

       buf = SUBSTR(buf,2)
     end
     else
      do
       brk = LENGTH(buf) + 1
       sp = POS(" ",buf)
       cm = POS(",",buf)
       rp = POS(")",buf)
       if sp <> 0 then
        brk = sp
       if cm <> 0 & cm < brk then
        brk = cm
       if rp <> 0 & rp < brk then
        do
         ltparens = 0
         brk = rp
       end
       brk = brk + 1
       extra = STRIP(SUBSTR(buf,1,brk - 1))
       buf = STRIP(SUBSTR(buf,brk))

       msgid = "0302I"
       issue_msg.0 = 2
       issue_msg.1 = extra
       issue_msg.2 = current_stmt

       call Write_Str
       QUEUE ' '
     end
     err_found = 1
   end
   OTHERWISE
    say "Unknown Error "sclparm" in ONLYIF"
  end
 END /* DO WHILE buf <> '' */

 /* Last keyword was missing end parenthesis */
 if ltparens <> 0 then
  do
   msgid = "0303I"
   issue_msg.0 = 3
   issue_msg.1 = sclparm
   issue_msg.2 = current_stmt
   issue_msg.3 = val

   call Write_Str

   msgid = "ADD"
   issue_msg.0 = 1
   issue_msg.1 = "Missing end parenthesis."

   call Write_Str
   QUEUE ' '

   ltparens = 0
   err_found = 1
 end

 /* Last keyword missing data */
 if keyword <> 0 then
  do
   msgid = "0306I"
   issue_msg.0 = 2
   issue_msg.1 = sclparm
   issue_msg.2 = current_stmt

   call Write_Str
   QUEUE ' '

   err_found = 1
 end
 return
/*-------------------------------------------------------------------*/
/* Control structures outlining valid keywords and the given syntax  */
/* for each keyword.                                                 */
/* The following is the structure for the struct:                    */
/* structs.ctrl_struct.0 --> Number of keywords for the statement    */
/* structs.ctrl_struct.x --> Name of the keyword                     */
/* structs.ctrl_struct.x.1 --> Abbreviation length of the keyword    */
/* structs.ctrlstruct.x.2.0 --> Number of keyword delimitters        */
/* structs.ctrl_struct.x.2.y --> Keyword delimitter                  */
/* structs.ctrl_struct.x.3 --> Value type                            */
/* structs.ctrl_struct.x.4 --> Value first character type            */
/* structs.ctrl_struct.x.5 --> Value min length                      */
/* structs.ctrl_struct.x.6 --> Value max length                      */
/* ------------------------------------------------------------------*/

/*-------------------------------------------------------------------*/
/* Make SETOAM control structure                                     */
/*-------------------------------------------------------------------*/
SETOAM_STRUCT:
 structs.ctrl_struct.0 = 20

 structs.ctrl_struct.1 = "SYSZTIOTUSEXTIOT"
 structs.ctrl_struct.1.1 = 12
 structs.ctrl_struct.1.2.0 = 1
 structs.ctrl_struct.1.2.1 = '('
 structs.ctrl_struct.1.3 = 'AN'
 structs.ctrl_struct.1.4 = ''
 structs.ctrl_struct.1.5 = 7
 structs.ctrl_struct.1.6 = 8

 structs.ctrl_struct.2 = "DISABLEOPENTYPEJ"
 structs.ctrl_struct.2.1 = 8
 structs.ctrl_struct.2.2.0 = 1
 structs.ctrl_struct.2.2.1 = '('
 structs.ctrl_struct.2.3 = 'AN'
 structs.ctrl_struct.2.4 = ''
 structs.ctrl_struct.2.5 = 2
 structs.ctrl_struct.2.6 = 5

 structs.ctrl_struct.3 = "MAXTAPERETRIEVETASKS"
 structs.ctrl_struct.3.1 = 8
 structs.ctrl_struct.3.2.0 = 1
 structs.ctrl_struct.3.2.1 = '('
 structs.ctrl_struct.3.3 = 'N'
 structs.ctrl_struct.3.4 = ''
 structs.ctrl_struct.3.5 = 1
 structs.ctrl_struct.3.6 = 3

 structs.ctrl_struct.4 = "MAXTAPESTORETASKS"
 structs.ctrl_struct.4.1 = 8
 structs.ctrl_struct.4.2.0 = 1
 structs.ctrl_struct.4.2.1 = '('
 structs.ctrl_struct.4.3 = 'N'
 structs.ctrl_struct.4.4 = ''
 structs.ctrl_struct.4.5 = 1
 structs.ctrl_struct.4.6 = 3

 structs.ctrl_struct.5 = "MOUNTWAITTIME"
 structs.ctrl_struct.5.1 = 6
 structs.ctrl_struct.5.2.0 = 1
 structs.ctrl_struct.5.2.1 = '('
 structs.ctrl_struct.5.3 = 'N'
 structs.ctrl_struct.5.4 = ''
 structs.ctrl_struct.5.5 = 1
 structs.ctrl_struct.5.6 = 3

 structs.ctrl_struct.6 = "DEMOUNTWAITTIME"
 structs.ctrl_struct.6.1 = 8
 structs.ctrl_struct.6.2.0 = 1
 structs.ctrl_struct.6.2.1 = '('
 structs.ctrl_struct.6.3 = 'N'
 structs.ctrl_struct.6.4 = ''
 structs.ctrl_struct.6.5 = 1
 structs.ctrl_struct.6.6 = 4

 structs.ctrl_struct.7 = "TAPESDB"
 structs.ctrl_struct.7.1 = 7
 structs.ctrl_struct.7.2.0 = 1
 structs.ctrl_struct.7.2.1 = '('
 structs.ctrl_struct.7.3 = 'AN'
 structs.ctrl_struct.7.4 = ''
 structs.ctrl_struct.7.5 = 5
 structs.ctrl_struct.7.6 = 5

 structs.ctrl_struct.8 = "ALLOCRETRYMINUTES"
 structs.ctrl_struct.8.1 = 8
 structs.ctrl_struct.8.2.0 = 1
 structs.ctrl_struct.8.2.1 = '('
 structs.ctrl_struct.8.3 = 'N'
 structs.ctrl_struct.8.4 = ''
 structs.ctrl_struct.8.5 = 1
 structs.ctrl_struct.8.6 = 1

 structs.ctrl_struct.9 = "TAPEFULLTHRESHOLD"
 structs.ctrl_struct.9.1 = 6
 structs.ctrl_struct.9.2.0 = 1
 structs.ctrl_struct.9.2.1 = '('
 structs.ctrl_struct.9.3 = 'N'
 structs.ctrl_struct.9.4 = ''
 structs.ctrl_struct.9.5 = 1
 structs.ctrl_struct.9.6 = 6

 structs.ctrl_struct.10 = "TAPEEXPIRATION"
 structs.ctrl_struct.10.1 = 6
 structs.ctrl_struct.10.2.0 = 1
 structs.ctrl_struct.10.2.1 = '('
 structs.ctrl_struct.10.3 = '' /* Check type in validate */
 structs.ctrl_struct.10.4 = ''
 structs.ctrl_struct.10.5 = 8
 structs.ctrl_struct.10.6 = 8

 structs.ctrl_struct.11 = "DSNWITHSGNAME"
 structs.ctrl_struct.11.1 = 9
 structs.ctrl_struct.11.2.0 = 3
 structs.ctrl_struct.11.2.1 = ' '
 structs.ctrl_struct.11.2.2 = ','
 structs.ctrl_struct.11.2.3 = ')'
 structs.ctrl_struct.11.3 = 'NONE'
 structs.ctrl_struct.11.4 = ''
 structs.ctrl_struct.11.5 = -1
 structs.ctrl_struct.11.6 = -1

 structs.ctrl_struct.12 = "DATACLASS"
 structs.ctrl_struct.12.1 = 5
 structs.ctrl_struct.12.2.0 = 1
 structs.ctrl_struct.12.2.1 = '('
 structs.ctrl_struct.12.3 = 'AN'
 structs.ctrl_struct.12.4 = 'A'
 structs.ctrl_struct.12.5 = 1
 structs.ctrl_struct.12.6 = 8

 structs.ctrl_struct.13 = "L2DATACLASS"
 structs.ctrl_struct.13.1 = 7
 structs.ctrl_struct.13.2.0 = 1
 structs.ctrl_struct.13.2.1 = '('
 structs.ctrl_struct.13.3 = 'AN'
 structs.ctrl_struct.13.4 = 'A'
 structs.ctrl_struct.13.5 = 1
 structs.ctrl_struct.13.6 = 8

 structs.ctrl_struct.14 = "STORAGEGROUP"
 structs.ctrl_struct.14.1 = 8
 structs.ctrl_struct.14.2.0 = 1
 structs.ctrl_struct.14.2.1 = '('
 structs.ctrl_struct.14.3 = 'AN'
 structs.ctrl_struct.14.4 = 'A'
 structs.ctrl_struct.14.5 = 1
 structs.ctrl_struct.14.6 = 8

 structs.ctrl_struct.15 = "TAPECAPACITY" /* Process in Validate */
 structs.ctrl_struct.15.1 = 6
 structs.ctrl_struct.15.2.0 = 1
 structs.ctrl_struct.15.2.1 = '('
 structs.ctrl_struct.15.3 = ''
 structs.ctrl_struct.15.4 = ''
 structs.ctrl_struct.15.5 = 1
 structs.ctrl_struct.15.6 = 8

 structs.ctrl_struct.16 = "TAPERECYCLEMODE"
 structs.ctrl_struct.16.1 = 6
 structs.ctrl_struct.16.2.0 = 1
 structs.ctrl_struct.16.2.1 = '('
 structs.ctrl_struct.16.3 = 'A'
 structs.ctrl_struct.16.4 = ''
 structs.ctrl_struct.16.5 = 5
 structs.ctrl_struct.16.6 = 10

 structs.ctrl_struct.17 = "OAMSCRATCHSYNCH"
 structs.ctrl_struct.17.1 = 13
 structs.ctrl_struct.17.2.0 = 1
 structs.ctrl_struct.17.2.1 = '('
 structs.ctrl_struct.17.3 = 'AN'
 structs.ctrl_struct.17.4 = ''
 structs.ctrl_struct.17.5 = 7
 structs.ctrl_struct.17.6 = 8

 structs.ctrl_struct.18 = "TAPEDISPATCHERDELAY"
 structs.ctrl_struct.18.1 = 7
 structs.ctrl_struct.18.2.0 = 1
 structs.ctrl_struct.18.2.1 = '('
 structs.ctrl_struct.18.3 = 'N'
 structs.ctrl_struct.18.4 = ''
 structs.ctrl_struct.18.5 = 1
 structs.ctrl_struct.18.6 = 2

 structs.ctrl_struct.19 = "MAXRECYCLETASKS"
 structs.ctrl_struct.19.1 = 8
 structs.ctrl_struct.19.2.0 = 1
 structs.ctrl_struct.19.2.1 = '('
 structs.ctrl_struct.19.3 = 'N'
 structs.ctrl_struct.19.4 = ''
 structs.ctrl_struct.19.5 = 1
 structs.ctrl_struct.19.6 = 2

 structs.ctrl_struct.20 = "PERCENTVALID"
 structs.ctrl_struct.20.1 = 8
 structs.ctrl_struct.20.2.0 = 1
 structs.ctrl_struct.20.2.1 = '('
 structs.ctrl_struct.20.3 = 'N'
 structs.ctrl_struct.20.4 = ''
 structs.ctrl_struct.20.5 = 1
 structs.ctrl_struct.20.6 = 3

 structs.sg_ctrl_struct.0 = 16

 structs.sg_ctrl_struct.1 = "TAPEUNITNAME"
 structs.sg_ctrl_struct.1.1 = 5
 structs.sg_ctrl_struct.1.2.0 = 1
 structs.sg_ctrl_struct.1.2.1 = '('
 structs.sg_ctrl_struct.1.3 = '' /* Check type in validate */
 structs.sg_ctrl_struct.1.4 = ''
 structs.sg_ctrl_struct.1.5 = 1
 structs.sg_ctrl_struct.1.6 = 8

 structs.sg_ctrl_struct.2 = "TAPECOMPACTION"
 structs.sg_ctrl_struct.2.1 = 6
 structs.sg_ctrl_struct.2.2.0 = 3
 structs.sg_ctrl_struct.2.2.1 = ' '
 structs.sg_ctrl_struct.2.2.2 = ','
 structs.sg_ctrl_struct.2.2.3 = ')'
 structs.sg_ctrl_struct.2.3 = 'NONE'
 structs.sg_ctrl_struct.2.4 = ''
 structs.sg_ctrl_struct.2.5 = -1
 structs.sg_ctrl_struct.2.6 = -1

 structs.sg_ctrl_struct.3 = "NOTAPECOMPACTION"
 structs.sg_ctrl_struct.3.1 = 8
 structs.sg_ctrl_struct.3.2.0 = 3
 structs.sg_ctrl_struct.3.2.1 = ' '
 structs.sg_ctrl_struct.3.2.2 = ','
 structs.sg_ctrl_struct.3.2.3 = ')'
 structs.sg_ctrl_struct.3.3 = 'NONE'
 structs.sg_ctrl_struct.3.4 = ''
 structs.sg_ctrl_struct.3.5 = -1
 structs.sg_ctrl_struct.3.6 = -1

 structs.sg_ctrl_struct.4 = "MAXTAPERETRIEVETASKS"
 structs.sg_ctrl_struct.4.1 = 8
 structs.sg_ctrl_struct.4.2.0 = 1
 structs.sg_ctrl_struct.4.2.1 = '('
 structs.sg_ctrl_struct.4.3 = 'N'
 structs.sg_ctrl_struct.4.4 = ''
 structs.sg_ctrl_struct.4.5 = 1
 structs.sg_ctrl_struct.4.6 = 3

 structs.sg_ctrl_struct.5 = "MAXTAPESTORETASKS"
 structs.sg_ctrl_struct.5.1 = 8
 structs.sg_ctrl_struct.5.2.0 = 1
 structs.sg_ctrl_struct.5.2.1 = '('
 structs.sg_ctrl_struct.5.3 = 'N'
 structs.sg_ctrl_struct.5.4 = ''
 structs.sg_ctrl_struct.5.5 = 1
 structs.sg_ctrl_struct.5.6 = 3

 structs.sg_ctrl_struct.6 = "SGMAXTAPERETRIEVETASKS"
 structs.sg_ctrl_struct.6.1 = 10
 structs.sg_ctrl_struct.6.2.0 = 1
 structs.sg_ctrl_struct.6.2.1 = '('
 structs.sg_ctrl_struct.6.3 = 'N'
 structs.sg_ctrl_struct.6.4 = ''
 structs.sg_ctrl_struct.6.5 = 1
 structs.sg_ctrl_struct.6.6 = 3

 structs.sg_ctrl_struct.7 = "SGMAXTAPESTORETASKS"
 structs.sg_ctrl_struct.7.1 = 10
 structs.sg_ctrl_struct.7.2.0 = 1
 structs.sg_ctrl_struct.7.2.1 = '('
 structs.sg_ctrl_struct.7.3 = 'N'
 structs.sg_ctrl_struct.7.4 = ''
 structs.sg_ctrl_struct.7.5 = 1
 structs.sg_ctrl_struct.7.6 = 3

 structs.sg_ctrl_struct.8 = "SGMAXRECYCLETASKS"
 structs.sg_ctrl_struct.8.1 = 8
 structs.sg_ctrl_struct.8.2.0 = 1
 structs.sg_ctrl_struct.8.2.1 = '('
 structs.sg_ctrl_struct.8.3 = 'N'
 structs.sg_ctrl_struct.8.4 = ''
 structs.sg_ctrl_struct.8.5 = 1
 structs.sg_ctrl_struct.8.6 = 2

 structs.sg_ctrl_struct.9 = "TAPEDRIVESTARTUP"
 structs.sg_ctrl_struct.9.1 = 6
 structs.sg_ctrl_struct.9.2.0 = 1
 structs.sg_ctrl_struct.9.2.1 = '('
 structs.sg_ctrl_struct.9.3 = 'N'
 structs.sg_ctrl_struct.9.4 = ''
 structs.sg_ctrl_struct.9.5 = 1
 structs.sg_ctrl_struct.9.6 = 4

 structs.sg_ctrl_struct.10 = "TAPEPERCENTFULL"
 structs.sg_ctrl_struct.10.1 = 6
 structs.sg_ctrl_struct.10.2.0 = 1
 structs.sg_ctrl_struct.10.2.1 = '('
 structs.sg_ctrl_struct.10.3 = 'N'
 structs.sg_ctrl_struct.10.4 = ''
 structs.sg_ctrl_struct.10.5 = 1
 structs.sg_ctrl_struct.10.6 = 3

 structs.sg_ctrl_struct.11 = "DEMOUNTWAITTIME"
 structs.sg_ctrl_struct.11.1 = 8
 structs.sg_ctrl_struct.11.2.0 = 1
 structs.sg_ctrl_struct.11.2.1 = '('
 structs.sg_ctrl_struct.11.3 = 'N'
 structs.sg_ctrl_struct.11.4 = ''
 structs.sg_ctrl_struct.11.5 = 1
 structs.sg_ctrl_struct.11.6 = 4

 structs.sg_ctrl_struct.12 = "TAPEFULLTHRESHOLD"
 structs.sg_ctrl_struct.12.1 = 6
 structs.sg_ctrl_struct.12.2.0 = 1
 structs.sg_ctrl_struct.12.2.1 = '('
 structs.sg_ctrl_struct.12.3 = 'N'
 structs.sg_ctrl_struct.12.4 = ''
 structs.sg_ctrl_struct.12.5 = 1
 structs.sg_ctrl_struct.12.6 = 6

 structs.sg_ctrl_struct.13 = "TAPEEXPIRATION"
 structs.sg_ctrl_struct.13.1 = 6
 structs.sg_ctrl_struct.13.2.0 = 1
 structs.sg_ctrl_struct.13.2.1 = '('
 structs.sg_ctrl_struct.13.3 = '' /* Check type in validate */
 structs.sg_ctrl_struct.13.4 = ''
 structs.sg_ctrl_struct.13.5 = 8
 structs.sg_ctrl_struct.13.6 = 8

 structs.sg_ctrl_struct.14 = "DATACLASS"
 structs.sg_ctrl_struct.14.1 = 5
 structs.sg_ctrl_struct.14.2.0 = 1
 structs.sg_ctrl_struct.14.2.1 = '('
 structs.sg_ctrl_struct.14.3 = 'AN'
 structs.sg_ctrl_struct.14.4 = 'A'
 structs.sg_ctrl_struct.14.5 = 1
 structs.sg_ctrl_struct.14.6 = 8

 structs.sg_ctrl_struct.15 = "L2DATACLASS"
 structs.sg_ctrl_struct.15.1 = 7
 structs.sg_ctrl_struct.15.2.0 = 1
 structs.sg_ctrl_struct.15.2.1 = '('
 structs.sg_ctrl_struct.15.3 = 'AN'
 structs.sg_ctrl_struct.15.4 = 'A'
 structs.sg_ctrl_struct.15.5 = 1
 structs.sg_ctrl_struct.15.6 = 8

 structs.sg_ctrl_struct.16 = "L2TAPEUNITNAME"
 structs.sg_ctrl_struct.16.1 = 7
 structs.sg_ctrl_struct.16.2.0 = 1
 structs.sg_ctrl_struct.16.2.1 = '('
 structs.sg_ctrl_struct.16.3 = '' /* Check type in validate */
 structs.sg_ctrl_struct.16.4 = ''
 structs.sg_ctrl_struct.16.5 = 1
 structs.sg_ctrl_struct.16.6 = 8

 return

/*-------------------------------------------------------------------*/
/* Make SETOPT control structure                                     */
/*-------------------------------------------------------------------*/
SETOPT_STRUCT:
 structs.ctrl_struct.0 = 17

 structs.ctrl_struct.1 = "OPTICALREINITMODE"
 structs.ctrl_struct.1.1 = 10
 structs.ctrl_struct.1.2.0 = 1
 structs.ctrl_struct.1.2.1 = '('
 structs.ctrl_struct.1.3 = 'A'
 structs.ctrl_struct.1.4 = 'A'
 structs.ctrl_struct.1.5 = 1
 structs.ctrl_struct.1.6 = 10

 structs.ctrl_struct.2 = "ABALL"
 structs.ctrl_struct.2.1 = 4
 structs.ctrl_struct.2.2.0 = 1
 structs.ctrl_struct.2.2.1 = '('
 structs.ctrl_struct.2.3 = 'AN'
 structs.ctrl_struct.2.4 = 'A'
 structs.ctrl_struct.2.5 = 7
 structs.ctrl_struct.2.6 = 8

 structs.ctrl_struct.3 = "ABDB2ERROR"
 structs.ctrl_struct.3.1 = 6
 structs.ctrl_struct.3.2.0 = 1
 structs.ctrl_struct.3.2.1 = '('
 structs.ctrl_struct.3.3 = 'AN'
 structs.ctrl_struct.3.4 = 'A'
 structs.ctrl_struct.3.5 = 7
 structs.ctrl_struct.3.6 = 8

 structs.ctrl_struct.4 = "ABLOST"
 structs.ctrl_struct.4.1 = 6
 structs.ctrl_struct.4.2.0 = 1
 structs.ctrl_struct.4.2.1 = '('
 structs.ctrl_struct.4.3 = 'AN'
 structs.ctrl_struct.4.4 = 'A'
 structs.ctrl_struct.4.5 = 7
 structs.ctrl_struct.4.6 = 8

 structs.ctrl_struct.5 = "ABNOTOPER"
 structs.ctrl_struct.5.1 = 6
 structs.ctrl_struct.5.2.0 = 1
 structs.ctrl_struct.5.2.1 = '('
 structs.ctrl_struct.5.3 = 'AN'
 structs.ctrl_struct.5.4 = 'A'
 structs.ctrl_struct.5.5 = 7
 structs.ctrl_struct.5.6 = 8

 structs.ctrl_struct.6 = "ABOFFLINE"
 structs.ctrl_struct.6.1 = 6
 structs.ctrl_struct.6.2.0 = 1
 structs.ctrl_struct.6.2.1 = '('
 structs.ctrl_struct.6.3 = 'AN'
 structs.ctrl_struct.6.4 = 'A'
 structs.ctrl_struct.6.5 = 7
 structs.ctrl_struct.6.6 = 8

 structs.ctrl_struct.7 = "ABUNREAD"
 structs.ctrl_struct.7.1 = 6
 structs.ctrl_struct.7.2.0 = 1
 structs.ctrl_struct.7.2.1 = '('
 structs.ctrl_struct.7.3 = 'AN'
 structs.ctrl_struct.7.4 = 'A'
 structs.ctrl_struct.7.5 = 7
 structs.ctrl_struct.7.6 = 8

 structs.ctrl_struct.8 = "ABFSERROR"
 structs.ctrl_struct.8.1 = 6
 structs.ctrl_struct.8.2.0 = 1
 structs.ctrl_struct.8.2.1 = '('
 structs.ctrl_struct.8.3 = 'AN'
 structs.ctrl_struct.8.4 = 'A'
 structs.ctrl_struct.8.5 = 7
 structs.ctrl_struct.8.6 = 8

 structs.ctrl_struct.9 = "ABCLDERROR"
 structs.ctrl_struct.9.1 = 6
 structs.ctrl_struct.9.2.0 = 1
 structs.ctrl_struct.9.2.1 = '('
 structs.ctrl_struct.9.3 = 'AN'
 structs.ctrl_struct.9.4 = 'A'
 structs.ctrl_struct.9.5 = 7
 structs.ctrl_struct.9.6 = 8

 structs.ctrl_struct.10 = "MULTISYSENABLE"
 structs.ctrl_struct.10.1 = 6
 structs.ctrl_struct.10.2.0 = 1
 structs.ctrl_struct.10.2.1 = '('
 structs.ctrl_struct.10.3 = 'AN'
 structs.ctrl_struct.10.4 = 'A'
 structs.ctrl_struct.10.5 = 1
 structs.ctrl_struct.10.6 = 3

 structs.ctrl_struct.11 = "OPTICALDISPATCHERDELAY"
 structs.ctrl_struct.11.1 = 10
 structs.ctrl_struct.11.2.0 = 1
 structs.ctrl_struct.11.2.1 = '('
 structs.ctrl_struct.11.3 = 'N'
 structs.ctrl_struct.11.4 = ''
 structs.ctrl_struct.11.5 = 1
 structs.ctrl_struct.11.6 = 2

 structs.ctrl_struct.12 = "SCRENTRYTHRESHOLD"
 structs.ctrl_struct.12.1 = 6
 structs.ctrl_struct.12.2.0 = 1
 structs.ctrl_struct.12.2.1 = '('
 structs.ctrl_struct.12.3 = 'N'
 structs.ctrl_struct.12.4 = ''
 structs.ctrl_struct.12.5 = 1
 structs.ctrl_struct.12.6 = 6

 structs.ctrl_struct.13 = "MOUNTWAITTIME"
 structs.ctrl_struct.13.1 = 6
 structs.ctrl_struct.13.2.0 = 1
 structs.ctrl_struct.13.2.1 = '('
 structs.ctrl_struct.13.3 = 'N'
 structs.ctrl_struct.13.4 = ''
 structs.ctrl_struct.13.5 = 1
 structs.ctrl_struct.13.6 = 3

 structs.ctrl_struct.14 = "UNLOADDRIVES"
 structs.ctrl_struct.14.1 = 7
 structs.ctrl_struct.14.2.0 = 1
 structs.ctrl_struct.14.2.1 = '('
 structs.ctrl_struct.14.3 = 'N'
 structs.ctrl_struct.14.4 = ''
 structs.ctrl_struct.14.5 = 1
 structs.ctrl_struct.14.6 = 1

 structs.ctrl_struct.15 = "UNLOADTIMER"
 structs.ctrl_struct.15.1 = 7
 structs.ctrl_struct.15.2.0 = 1
 structs.ctrl_struct.15.2.1 = '('
 structs.ctrl_struct.15.3 = 'N'
 structs.ctrl_struct.15.4 = ''
 structs.ctrl_struct.15.5 = 1
 structs.ctrl_struct.15.6 = 4

 structs.ctrl_struct.16 = "STORAGEGROUP"
 structs.ctrl_struct.16.1 = 8
 structs.ctrl_struct.16.2.0 = 1
 structs.ctrl_struct.16.2.1 = '('
 structs.ctrl_struct.16.3 = 'AN'
 structs.ctrl_struct.16.4 = 'A'
 structs.ctrl_struct.16.5 = 1
 structs.ctrl_struct.16.6 = 8

 structs.ctrl_struct.17 = "DB2RECONNECTWAITTIME"
 structs.ctrl_struct.17.1 = 8
 structs.ctrl_struct.17.2.0 = 1
 structs.ctrl_struct.17.2.1 = '('
 structs.ctrl_struct.17.3 = 'N'
 structs.ctrl_struct.17.4 = ''
 structs.ctrl_struct.17.5 = 1
 structs.ctrl_struct.17.6 = 5

 structs.sg_ctrl_struct.0 = 1

 structs.sg_ctrl_struct.1 = "OPTICALREINITMODE"
 structs.sg_ctrl_struct.1.1 = 10
 structs.sg_ctrl_struct.1.2.0 = 1
 structs.sg_ctrl_struct.1.2.1 = '('
 structs.sg_ctrl_struct.1.3 = 'A'
 structs.sg_ctrl_struct.1.4 = 'A'
 structs.sg_ctrl_struct.1.5 = 1
 structs.sg_ctrl_struct.1.6 = 10

 return

/*-------------------------------------------------------------------*/
/* Make OAMXCF control structure                                     */
/*-------------------------------------------------------------------*/
OAMXCF_STRUCT:
 structs.ctrl_struct.0 = 3

 structs.ctrl_struct.1 = "XCFTIMEOUT"
 structs.ctrl_struct.1.1 = 8
 structs.ctrl_struct.1.2.0 = 1
 structs.ctrl_struct.1.2.1 = '('
 structs.ctrl_struct.1.3 = 'NONE'
 structs.ctrl_struct.1.4 = ''
 structs.ctrl_struct.1.5 = -1
 structs.ctrl_struct.1.6 = -1

 structs.ctrl_struct.2 = "OAMGROUPNAME"
 structs.ctrl_struct.2.1 = 8
 structs.ctrl_struct.2.2.0 = 1
 structs.ctrl_struct.2.2.1 = '('
 structs.ctrl_struct.2.3 = 'AN'
 structs.ctrl_struct.2.4 = 'A'
 structs.ctrl_struct.2.5 = 1
 structs.ctrl_struct.2.6 = 8

 structs.ctrl_struct.3 = "OAMMEMBERNAME"
 structs.ctrl_struct.3.1 = 6
 structs.ctrl_struct.3.2.0 = 1
 structs.ctrl_struct.3.2.1 = '('
 structs.ctrl_struct.3.3 = 'AN'
 structs.ctrl_struct.3.4 = 'A'
 structs.ctrl_struct.3.5 = 1
 structs.ctrl_struct.3.6 = 16

 structs.sg_ctrl_struct.0 = 6

 structs.sg_ctrl_struct.1 = "XCFOPTREADA"
 structs.sg_ctrl_struct.1.1 = 11
 structs.sg_ctrl_struct.1.2.0 = 1
 structs.sg_ctrl_struct.1.2.1 = '('
 structs.sg_ctrl_struct.1.3 = 'N'
 structs.sg_ctrl_struct.1.4 = ''
 structs.sg_ctrl_struct.1.5 = 1
 structs.sg_ctrl_struct.1.6 = 6

 structs.sg_ctrl_struct.2 = "XCFOPTREADM"
 structs.sg_ctrl_struct.2.1 = 11
 structs.sg_ctrl_struct.2.2.0 = 1
 structs.sg_ctrl_struct.2.2.1 = '('
 structs.sg_ctrl_struct.2.3 = 'N'
 structs.sg_ctrl_struct.2.4 = ''
 structs.sg_ctrl_struct.2.5 = 1
 structs.sg_ctrl_struct.2.6 = 6

 structs.sg_ctrl_struct.3 = "XCFOPTWRITEA"
 structs.sg_ctrl_struct.3.1 = 12
 structs.sg_ctrl_struct.3.2.0 = 1
 structs.sg_ctrl_struct.3.2.1 = '('
 structs.sg_ctrl_struct.3.3 = 'N'
 structs.sg_ctrl_struct.3.4 = ''
 structs.sg_ctrl_struct.3.5 = 1
 structs.sg_ctrl_struct.3.6 = 6

 structs.sg_ctrl_struct.4 = "XCFOPTWRITEM"
 structs.sg_ctrl_struct.4.1 = 12
 structs.sg_ctrl_struct.4.2.0 = 1
 structs.sg_ctrl_struct.4.2.1 = '('
 structs.sg_ctrl_struct.4.3 = 'N'
 structs.sg_ctrl_struct.4.4 = ''
 structs.sg_ctrl_struct.4.5 = 1
 structs.sg_ctrl_struct.4.6 = 6

 structs.sg_ctrl_struct.5 = "XCFTAPEREADA"
 structs.sg_ctrl_struct.5.1 = 12
 structs.sg_ctrl_struct.5.2.0 = 1
 structs.sg_ctrl_struct.5.2.1 = '('
 structs.sg_ctrl_struct.5.3 = 'N'
 structs.sg_ctrl_struct.5.4 = ''
 structs.sg_ctrl_struct.5.5 = 1
 structs.sg_ctrl_struct.5.6 = 6

 structs.sg_ctrl_struct.6 = "XCFTAPEREADM"
 structs.sg_ctrl_struct.6.1 = 12
 structs.sg_ctrl_struct.6.2.0 = 1
 structs.sg_ctrl_struct.6.2.1 = '('
 structs.sg_ctrl_struct.6.3 = 'N'
 structs.sg_ctrl_struct.6.4 = ''
 structs.sg_ctrl_struct.6.5 = 1
 structs.sg_ctrl_struct.6.6 = 6

 return

/*-------------------------------------------------------------------*/
/* Make SETOSMC control structure                                    */
/*-------------------------------------------------------------------*/
SETOSMC_STRUCT:
 structs.ctrl_struct.0 = 14

 structs.ctrl_struct.1 = "FIRSTBACKUPGROUP"
 structs.ctrl_struct.1.1 = 6
 structs.ctrl_struct.1.2.0 = 1
 structs.ctrl_struct.1.2.1 = '('
 structs.ctrl_struct.1.3 = 'AN'
 structs.ctrl_struct.1.4 = 'A'
 structs.ctrl_struct.1.5 = 1
 structs.ctrl_struct.1.6 = 8

 structs.ctrl_struct.2 = "SECONDBACKUPGROUP"
 structs.ctrl_struct.2.1 = 7
 structs.ctrl_struct.2.2.0 = 1
 structs.ctrl_struct.2.2.1 = '('
 structs.ctrl_struct.2.3 = 'AN'
 structs.ctrl_struct.2.4 = ''
 structs.ctrl_struct.2.5 = 1
 structs.ctrl_struct.2.6 = 8

 structs.ctrl_struct.3 = "CYCLEWINDOW"
 structs.ctrl_struct.3.1 = 6
 structs.ctrl_struct.3.2.0 = 1
 structs.ctrl_struct.3.2.1 = '('
 structs.ctrl_struct.3.3 = 'A'
 structs.ctrl_struct.3.4 = 'A'
 structs.ctrl_struct.3.5 = 9
 structs.ctrl_struct.3.6 = 9

 structs.ctrl_struct.4 = "CLEAROLDLOC"
 structs.ctrl_struct.4.1 = 6
 structs.ctrl_struct.4.2.0 = 1
 structs.ctrl_struct.4.2.1 = '('
 structs.ctrl_struct.4.3 = 'A'
 structs.ctrl_struct.4.4 = ''
 structs.ctrl_struct.4.5 = 3
 structs.ctrl_struct.4.6 = 4

 structs.ctrl_struct.5 = "MAXRECALLTASKS"
 structs.ctrl_struct.5.1 = 10
 structs.ctrl_struct.5.2.0 = 1
 structs.ctrl_struct.5.2.1 = '('
 structs.ctrl_struct.5.3 = 'N'
 structs.ctrl_struct.5.4 = ''
 structs.ctrl_struct.5.5 = 1
 structs.ctrl_struct.5.6 = 3

 structs.ctrl_struct.6 = "MAXOSMCFAIL"
 structs.ctrl_struct.6.1 = 8
 structs.ctrl_struct.6.2.0 = 1
 structs.ctrl_struct.6.2.1 = '('
 structs.ctrl_struct.6.3 = 'N'
 structs.ctrl_struct.6.4 = ''
 structs.ctrl_struct.6.5 = 1
 structs.ctrl_struct.6.6 = 3

 structs.ctrl_struct.7 = "RECALLCLOUD"
 structs.ctrl_struct.7.1 = 7
 structs.ctrl_struct.7.2.0 = 1
 structs.ctrl_struct.7.2.1 = '('
 structs.ctrl_struct.7.3 = 'N'
 structs.ctrl_struct.7.4 = ''
 structs.ctrl_struct.7.5 = 1
 structs.ctrl_struct.7.6 = 3

 structs.ctrl_struct.8 = "RECALLOPTICAL"
 structs.ctrl_struct.8.1 = 8
 structs.ctrl_struct.8.2.0 = 1
 structs.ctrl_struct.8.2.1 = '('
 structs.ctrl_struct.8.3 = 'N'
 structs.ctrl_struct.8.4 = ''
 structs.ctrl_struct.8.5 = 1
 structs.ctrl_struct.8.6 = 3

 structs.ctrl_struct.9 = "RECALLTAPE"
 structs.ctrl_struct.9.1 = 7
 structs.ctrl_struct.9.2.0 = 1
 structs.ctrl_struct.9.2.1 = '('
 structs.ctrl_struct.9.3 = 'N'
 structs.ctrl_struct.9.4 = ''
 structs.ctrl_struct.9.5 = 1
 structs.ctrl_struct.9.6 = 3

 structs.ctrl_struct.10 = "RECALLNONE"
 structs.ctrl_struct.10.1 = 7
 structs.ctrl_struct.10.2.0 = 3
 structs.ctrl_struct.10.2.1 = ' '
 structs.ctrl_struct.10.2.1 = ','
 structs.ctrl_struct.10.2.1 = ')'
 structs.ctrl_struct.10.3 = 'NONE'
 structs.ctrl_struct.10.4 = ''
 structs.ctrl_struct.10.5 = -1
 structs.ctrl_struct.10.6 = -1

 structs.ctrl_struct.11 = "RECALLALL"
 structs.ctrl_struct.11.1 = 7
 structs.ctrl_struct.11.2.0 = 1
 structs.ctrl_struct.11.2.1 = '('
 structs.ctrl_struct.11.3 = 'N'
 structs.ctrl_struct.11.4 = ''
 structs.ctrl_struct.11.5 = 1
 structs.ctrl_struct.11.6 = 3

 structs.ctrl_struct.12 = "RECALLDISKSUBLEVEL"
 structs.ctrl_struct.12.1 = 7
 structs.ctrl_struct.12.2.0 = 1
 structs.ctrl_struct.12.2.1 = '('
 structs.ctrl_struct.12.3 = 'N'
 structs.ctrl_struct.12.4 = ''
 structs.ctrl_struct.12.5 = 1
 structs.ctrl_struct.12.6 = 1

 structs.ctrl_struct.13 = "BACKUPDELETE"
 structs.ctrl_struct.13.1 = 8
 structs.ctrl_struct.13.2.0 = 1
 structs.ctrl_struct.13.2.1 = '('
 structs.ctrl_struct.13.3 = 'A'
 structs.ctrl_struct.13.4 = ''
 structs.ctrl_struct.13.5 = 2
 structs.ctrl_struct.13.6 = 3

 structs.ctrl_struct.14 = "STORAGEGROUP"
 structs.ctrl_struct.14.1 = 8
 structs.ctrl_struct.14.2.0 = 1
 structs.ctrl_struct.14.2.1 = '('
 structs.ctrl_struct.14.3 = 'AN'
 structs.ctrl_struct.14.4 = 'A'
 structs.ctrl_struct.14.5 = 1
 structs.ctrl_struct.14.6 = 8

 structs.sg_ctrl_struct.0 = 11

 structs.sg_ctrl_struct.1 = "FIRSTBACKUPGROUP"
 structs.sg_ctrl_struct.1.1 = 6
 structs.sg_ctrl_struct.1.2.0 = 1
 structs.sg_ctrl_struct.1.2.1 = '('
 structs.sg_ctrl_struct.1.3 = 'AN'
 structs.sg_ctrl_struct.1.4 = 'A'
 structs.sg_ctrl_struct.1.5 = 1
 structs.sg_ctrl_struct.1.6 = 8

 structs.sg_ctrl_struct.2 = "SECONDBACKUPGROUP"
 structs.sg_ctrl_struct.2.1 = 7
 structs.sg_ctrl_struct.2.2.0 = 1
 structs.sg_ctrl_struct.2.2.1 = '('
 structs.sg_ctrl_struct.2.3 = 'AN'
 structs.sg_ctrl_struct.2.4 = 'A'
 structs.sg_ctrl_struct.2.5 = 1
 structs.sg_ctrl_struct.2.6 = 8

 structs.sg_ctrl_struct.3 = "CLEAROLDLOC"
 structs.sg_ctrl_struct.3.1 = 6
 structs.sg_ctrl_struct.3.2.0 = 1
 structs.sg_ctrl_struct.3.2.1 = '('
 structs.sg_ctrl_struct.3.3 = 'AN'
 structs.sg_ctrl_struct.3.4 = 'A'
 structs.sg_ctrl_struct.3.5 = 3
 structs.sg_ctrl_struct.3.6 = 4

 structs.sg_ctrl_struct.4 = "RECALLNONE"
 structs.sg_ctrl_struct.4.1 = 7
 structs.sg_ctrl_struct.4.2.0 = 3
 structs.sg_ctrl_struct.4.2.1 = ' '
 structs.sg_ctrl_struct.4.2.1 = ','
 structs.sg_ctrl_struct.4.2.1 = ')'
 structs.sg_ctrl_struct.4.3 = 'NONE'
 structs.sg_ctrl_struct.4.4 = ''
 structs.sg_ctrl_struct.4.5 = -1
 structs.sg_ctrl_struct.4.6 = -1

 structs.sg_ctrl_struct.5 = "RECALLCLOUD"
 structs.sg_ctrl_struct.5.1 = 7
 structs.sg_ctrl_struct.5.2.0 = 1
 structs.sg_ctrl_struct.5.2.1 = '('
 structs.sg_ctrl_struct.5.3 = 'N'
 structs.sg_ctrl_struct.5.4 = ''
 structs.sg_ctrl_struct.5.5 = 1
 structs.sg_ctrl_struct.5.6 = 3

 structs.sg_ctrl_struct.6 = "RECALLOPTICAL"
 structs.sg_ctrl_struct.6.1 = 8
 structs.sg_ctrl_struct.6.2.0 = 1
 structs.sg_ctrl_struct.6.2.1 = '('
 structs.sg_ctrl_struct.6.3 = 'N'
 structs.sg_ctrl_struct.6.4 = ''
 structs.sg_ctrl_struct.6.5 = 1
 structs.sg_ctrl_struct.6.6 = 3

 structs.sg_ctrl_struct.7 = "RECALLTAPE"
 structs.sg_ctrl_struct.7.1 = 7
 structs.sg_ctrl_struct.7.2.0 = 1
 structs.sg_ctrl_struct.7.2.1 = '('
 structs.sg_ctrl_struct.7.3 = 'N'
 structs.sg_ctrl_struct.7.4 = ''
 structs.sg_ctrl_struct.7.5 = 1
 structs.sg_ctrl_struct.7.6 = 3

 structs.sg_ctrl_struct.8 = "RECALLALL"
 structs.sg_ctrl_struct.8.1 = 7
 structs.sg_ctrl_struct.8.2.0 = 1
 structs.sg_ctrl_struct.8.2.1 = '('
 structs.sg_ctrl_struct.8.3 = 'N'
 structs.sg_ctrl_struct.8.4 = ''
 structs.sg_ctrl_struct.8.5 = 1
 structs.sg_ctrl_struct.8.6 = 3

 structs.sg_ctrl_struct.9 = "RECALLOFF"
 structs.sg_ctrl_struct.9.1 = 8
 structs.sg_ctrl_struct.9.2.0 = 1
 structs.sg_ctrl_struct.9.2.1 = '('
 structs.sg_ctrl_struct.9.3 = 'A'
 structs.sg_ctrl_struct.9.4 = ''
 structs.sg_ctrl_struct.9.5 = 2
 structs.sg_ctrl_struct.9.6 = 3

 structs.sg_ctrl_struct.10 = "RECALLDISKSUBLEVEL"
 structs.sg_ctrl_struct.10.1 = 7
 structs.sg_ctrl_struct.10.2.0 = 1
 structs.sg_ctrl_struct.10.2.1 = '('
 structs.sg_ctrl_struct.10.3 = 'N'
 structs.sg_ctrl_struct.10.4 = ''
 structs.sg_ctrl_struct.10.5 = 1
 structs.sg_ctrl_struct.10.6 = 1

 structs.sg_ctrl_struct.11 = "BACKUPTIER"
 structs.sg_ctrl_struct.11.1 = 8
 structs.sg_ctrl_struct.11.2.0 = 1
 structs.sg_ctrl_struct.11.2.1 = '('
 structs.sg_ctrl_struct.11.3 = 'A'
 structs.sg_ctrl_struct.11.4 = ''
 structs.sg_ctrl_struct.11.5 = 2
 structs.sg_ctrl_struct.11.6 = 7

 return

/*-------------------------------------------------------------------*/
/* Make SETDISK control structure                                    */
/*-------------------------------------------------------------------*/
SETDISK_STRUCT:
 structs.ctrl_struct.0 = 1

 structs.ctrl_struct.1 = "STORAGEGROUP"
 structs.ctrl_struct.1.1 = 8
 structs.ctrl_struct.1.2.0 = 1
 structs.ctrl_struct.1.2.1 = '('
 structs.ctrl_struct.1.3 = 'AN'
 structs.ctrl_struct.1.4 = 'A'
 structs.ctrl_struct.1.5 = 1
 structs.ctrl_struct.1.6 = 8

 structs.sg_ctrl_struct.0 = 2

 structs.sg_ctrl_struct.1 = "L2TYPE"
 structs.sg_ctrl_struct.1.1 = 3
 structs.sg_ctrl_struct.1.2.0 = 1
 structs.sg_ctrl_struct.1.2.1 = '('
 structs.sg_ctrl_struct.1.3 = 'AN'
 structs.sg_ctrl_struct.1.4 = 'AN'
 structs.sg_ctrl_struct.1.5 = 1
 structs.sg_ctrl_struct.1.6 = 8

 structs.sg_ctrl_struct.2 = "L2DIR"
 structs.sg_ctrl_struct.2.1 = 3
 structs.sg_ctrl_struct.2.2.0 = 1
 structs.sg_ctrl_struct.2.2.1 = '('
 structs.sg_ctrl_struct.2.3 = 'l2dir_validchars'
 structs.sg_ctrl_struct.2.4 = ''
 structs.sg_ctrl_struct.2.5 = 1
 structs.sg_ctrl_struct.2.6 = 30

 return

/*-------------------------------------------------------------------*/
/* Make SETTLIB control structure                                    */
/*-------------------------------------------------------------------*/
SETTLIB_STRUCT:
 structs.ctrl_struct.0 = 7 /* number of keywords */

 structs.ctrl_struct.1 = "LACSRETRYFAIL"
 structs.ctrl_struct.1.1 = 10
 structs.ctrl_struct.1.2.0 = 1
 structs.ctrl_struct.1.2.1 = '('
 structs.ctrl_struct.1.3 = 'AN'
 structs.ctrl_struct.1.4 = 'A'
 structs.ctrl_struct.1.5 = 2
 structs.ctrl_struct.1.6 = 3

 structs.ctrl_struct.2 = "LACSRETRYMINUTES"
 structs.ctrl_struct.2.1 = 10
 structs.ctrl_struct.2.2.0 = 1
 structs.ctrl_struct.2.2.1 = '('
 structs.ctrl_struct.2.3 = 'N'
 structs.ctrl_struct.2.4 = ''
 structs.ctrl_struct.2.5 = 1
 structs.ctrl_struct.2.6 = 1

 structs.ctrl_struct.3 = "LACSRETRYTIMES"
 structs.ctrl_struct.3.1 = 10
 structs.ctrl_struct.3.2.0 = 1
 structs.ctrl_struct.3.2.1 = '('
 structs.ctrl_struct.3.3 = 'N'
 structs.ctrl_struct.3.4 = ''
 structs.ctrl_struct.3.5 = 1
 structs.ctrl_struct.3.6 = 1

 structs.ctrl_struct.4 = "SCRATCHTHRESHOLDMSG"
 structs.ctrl_struct.4.1 = 17
 structs.ctrl_struct.4.2.0 = 1
 structs.ctrl_struct.4.2.1 = '('
 structs.ctrl_struct.4.3 = 'N'
 structs.ctrl_struct.4.4 = ''
 structs.ctrl_struct.4.5 = 1
 structs.ctrl_struct.4.6 = 3

 structs.ctrl_struct.5 = "ENTRYIGNOREMSGTYPE"
 structs.ctrl_struct.5.1 = 15
 structs.ctrl_struct.5.2.0 = 1
 structs.ctrl_struct.5.2.1 = '('
 structs.ctrl_struct.5.3 = 'AN'
 structs.ctrl_struct.5.4 = 'A'
 structs.ctrl_struct.5.5 = 1
 structs.ctrl_struct.5.6 = 8

 structs.ctrl_struct.6 = "ENTRYIGNOREMSGDEST"
 structs.ctrl_struct.6.1 = 15
 structs.ctrl_struct.6.2.0 = 1
 structs.ctrl_struct.6.2.1 = '('
 structs.ctrl_struct.6.3 = 'AN'
 structs.ctrl_struct.6.4 = 'A'
 structs.ctrl_struct.6.5 = 1
 structs.ctrl_struct.6.6 = 8

 structs.ctrl_struct.7 = "ENTRYSUCCESSMSGDEST"
 structs.ctrl_struct.7.1 = 16
 structs.ctrl_struct.7.2.0 = 1
 structs.ctrl_struct.7.2.1 = '('
 structs.ctrl_struct.7.3 = 'AN'
 structs.ctrl_struct.7.4 = 'A'
 structs.ctrl_struct.7.5 = 1
 structs.ctrl_struct.7.6 = 8

 return

/*-------------------------------------------------------------------*/
/* Make SETCLOUD control structure                                   */
/*-------------------------------------------------------------------*/
SETCLOUD_STRUCT:
 structs.ctrl_struct.0 = 3

 structs.ctrl_struct.1 = "PROVIDER"
 structs.ctrl_struct.1.1 = 4
 structs.ctrl_struct.1.2.0 = 1
 structs.ctrl_struct.1.2.1 = '('
 structs.ctrl_struct.1.3 = 'AN'
 structs.ctrl_struct.1.4 = ''
 structs.ctrl_struct.1.5 = 1
 structs.ctrl_struct.1.6 = 20

 structs.ctrl_struct.2 = "CONTAINER"
 structs.ctrl_struct.2.1 = 4
 structs.ctrl_struct.2.2.0 = 1
 structs.ctrl_struct.2.2.1 = '('
 structs.ctrl_struct.2.3 = ''
 structs.ctrl_struct.2.4 = ''
 structs.ctrl_struct.2.5 = 3
 structs.ctrl_struct.2.6 = 63

 structs.ctrl_struct.3 = "STORAGEGROUP"
 structs.ctrl_struct.3.1 = 8
 structs.ctrl_struct.3.2.0 = 1
 structs.ctrl_struct.3.2.1 = '('
 structs.ctrl_struct.3.3 = 'AN'
 structs.ctrl_struct.3.4 = 'AN'
 structs.ctrl_struct.3.5 = 1
 structs.ctrl_struct.3.6 = 8

 structs.sg_ctrl_struct.0 = 2

 structs.sg_ctrl_struct.1 = "PROVIDER"
 structs.sg_ctrl_struct.1.1 = 4
 structs.sg_ctrl_struct.1.2.0 = 1
 structs.sg_ctrl_struct.1.2.1 = '('
 structs.sg_ctrl_struct.1.3 = 'AN'
 structs.sg_ctrl_struct.1.4 = ''
 structs.sg_ctrl_struct.1.5 = 1
 structs.sg_ctrl_struct.1.6 = 20

 structs.sg_ctrl_struct.2 = "CONTAINER"
 structs.sg_ctrl_struct.2.1 = 4
 structs.sg_ctrl_struct.2.2.0 = 1
 structs.sg_ctrl_struct.2.2.1 = '('
 structs.sg_ctrl_struct.2.3 = 'cont_validchars'
 structs.sg_ctrl_struct.2.4 = ''
 structs.sg_ctrl_struct.2.5 = 3
 structs.sg_ctrl_struct.2.6 = 63

 return

/*-------------------------------------------------------------------*/
/* Make ONLYIF control structure                                     */
/*-------------------------------------------------------------------*/
ONLYIF_STRUCT:
 structs.ctrl_struct.0 = 3

 structs.ctrl_struct.1 = "SYSNAME"
 structs.ctrl_struct.1.1 = 4
 structs.ctrl_struct.1.2.0 = 1
 structs.ctrl_struct.1.2.1 = '('
 structs.ctrl_struct.1.3 = 'onlyif_validchars'
 structs.ctrl_struct.1.4 = ''
 structs.ctrl_struct.1.5 = 1
 structs.ctrl_struct.1.6 = 8

 structs.ctrl_struct.2 = "DB2ID"
 structs.ctrl_struct.2.1 = 3
 structs.ctrl_struct.2.2.0 = 1
 structs.ctrl_struct.2.2.1 = '('
 structs.ctrl_struct.2.3 = 'onlyif_validchars'
 structs.ctrl_struct.2.4 = ''
 structs.ctrl_struct.2.5 = 1
 structs.ctrl_struct.2.6 = 5

 structs.ctrl_struct.3 = "OAMVAR"
 structs.ctrl_struct.3.1 = 4
 structs.ctrl_struct.3.2.0 = 1
 structs.ctrl_struct.3.2.1 = '('
 structs.ctrl_struct.3.3 = 'onlyif_validchars'
 structs.ctrl_struct.3.4 = ''
 structs.ctrl_struct.3.5 = 1
 structs.ctrl_struct.3.6 = 8

 return

/*-------------------------------------------------------------------*/
/* Make message list                                                 */
/*-------------------------------------------------------------------*/
Message_List:
 CBR.0302I = "CBR0302I Keyword *insert* invalid in a *insert* statement."

 CBR.0303I = "CBR0303I Data for keyword *insert* in a *insert* " ||,
             "statement is invalid - *insert*."

 CBR.0304I = "CBR0304I Extra data for keyword *insert* in a *insert* " ||,
             "statement has been found - *insert*."

 CBR.0305I = "CBR0305I STORAGEGROUP *insert* missing ending parenthesis in " ||,
             "a *insert* statement."

 CBR.0306I = "CBR0306I Data for keyword *insert* is missing in a *insert* " ||,
             "statement."

 CBR.0311I = "CBR0311I A *insert* subparameter has not been specified, or " ||,
             "is invalid, for STORAGEGROUP *insert*."

 CBR.0313I = "CBR0313I STORAGEGROUP *insert* *insert* value (*insert*) " ||,
             "is greater than SETOAM MAXTAPERETRIEVETASKS value (*insert*)."

 CBR.0314I = "CBR0314I STORAGEGROUP *insert* *insert* value (*insert*) " ||,
             "is greater than SETOAM MAXTAPESTORETASKS value (*insert*)."

 CBR.0319I = "CBR0319I *insert* command encountered in PARMLIB member " ||,
             "*insert* with no keywords."

 CBR.0326I = "CBR0326I XCFTIMEOUT parameter missing ending parenthesis."

 CBR.0329I = "CBR0329I PARMLIB member *insert* contains no valid " ||,
             "OAMGROUPNAME parameter for the OAMXCF command."

 CBR.0330I = "CBR0330I PARMLIB member *insert* contains no valid " ||,
             "OAMMEMBERNAME parameter for the OAMXCF command."

 CBR.0335I = "CBR0335I Group *insert* specified as both a FIRSTBACKUPGROUP " ||,
             "and a SECONDBACKUPGROUP in PARMLIB member *insert*."

 CBR.0336I = "CBR0336I Group *insert*, specified as either a " ||,
             "FIRSTBACKUPGROUP or a SECONDBACKUPGROUP in PARMLIB member " ||,
             "*insert*, is not an object backup storage group."

 CBR.0337I = "CBR0337I All object backup storage groups in configuration " ||,
             "defined in SECONDBACKUPGROUP statements in *insert* member " ||,
             "of PARMLIB. No object backup storage groups available to be " ||,
             "used as a FIRSTBACKUPGROUP."

 CBR.0342I = "CBR0342I STORAGEGROUP *insert* SGMAXRECYCLETASKS value " ||,
             "(*insert*) is greater than SETOAM MAXRECYCLETASKS value " ||,
             "(*insert*)."

 CBR.0344I = "CBR0344I Keyword *insert* invalid when specified with " ||,
             "storage group *insert* in a *insert* statement."

 CBR.0349I = "CBR0349I Storage group *insert* did not specify both L2TYPE " ||,
             "and L2DIR in the SETDISK statement."

 CBR.0362I = "CBR0362I *insert* specification for storage group *insert* " ||,
             "does not include both *insert* and *insert*."

 CBR.0363I = "CBR0363I GLOBAL specifications for *insert* keywords " ||,
             "*insert* and *insert* must both be provided if either is."

 CBR.WARN1 = "WARNING: Data for keyword *insert* in a *insert* " ||,
              "statement cannot be *insert*. Value will default to *insert*."

 CBR.ADD   = "*insert*"

 return
